"""
队列服务层
处理队列相关的业务逻辑
"""
from typing import Optional, List, Dict, Any
from datetime import datetime, timedelta, timezone
from sqlalchemy import text, select, func, and_
from sqlalchemy.ext.asyncio import AsyncSession
import logging

from jettask.db.models.task import Task
from jettask.db.models.task_run import TaskRun
from jettask.db.models.task_metrics_minute import TaskMetricsMinute
from jettask.db.models.queue_snapshot import QueueSnapshot
from jettask.webui.services.base_service import BaseService, TimeRangeResult, QueueResolutionResult
import base64
import json

logger = logging.getLogger(__name__)


class QueryConditionsResult:
    """查询条件构建结果"""
    def __init__(
        self,
        conditions: list,
        all_priority_queues: List[str],
        time_range_result: TimeRangeResult
    ):
        self.conditions = conditions
        self.all_priority_queues = all_priority_queues
        self.time_range_result = time_range_result


class QueueService(BaseService):
    """队列服务类"""
    
    @staticmethod
    def get_base_queue_name(queue_name: str) -> str:
        if ':' in queue_name:
            parts = queue_name.rsplit(':', 1)
            if parts[-1].isdigit():
                return parts[0]
        return queue_name
    
    @staticmethod
    async def get_queues_by_namespace(namespace_data_access, namespace: str) -> Dict[str, Any]:
        queues_data = await namespace_data_access.get_queue_stats(namespace)
        return {
            "success": True,
            "data": list(set([QueueService.get_base_queue_name(q['queue_name']) for q in queues_data]))
        }
    
    @staticmethod
    async def get_queue_flow_rates(data_access, query) -> Dict[str, Any]:
        now = datetime.now(timezone.utc)
        
        if query.start_time and query.end_time:
            start_time = query.start_time
            end_time = query.end_time
            logger.info(f"使用自定义时间范围: {start_time} 到 {end_time}")
        else:
            time_range_map = {
                "15m": timedelta(minutes=15),
                "30m": timedelta(minutes=30),
                "1h": timedelta(hours=1),
                "3h": timedelta(hours=3),
                "6h": timedelta(hours=6),
                "12h": timedelta(hours=12),
                "24h": timedelta(hours=24),
                "7d": timedelta(days=7),
                "30d": timedelta(days=30),
            }
            
            time_range_value = query.time_range if query.time_range else query.interval
            delta = time_range_map.get(time_range_value, timedelta(minutes=15))
            
            queue_name = query.queues[0] if query.queues else None
            if queue_name:
                latest_time = await data_access.get_latest_task_time(queue_name)
                if latest_time:
                    end_time = latest_time.replace(second=59, microsecond=999999)  
                    logger.info(f"使用最新任务时间: {latest_time}")
                else:
                    end_time = now.replace(second=0, microsecond=0)
            else:
                end_time = now.replace(second=0, microsecond=0)
            
            start_time = end_time - delta
            logger.info(f"使用预设时间范围 {time_range_value}: {start_time} 到 {end_time}, delta: {delta}")
        
        if not query.queues or len(query.queues) == 0:
            return {"data": [], "granularity": "minute"}
        
        queue_name = query.queues[0]
        filters = getattr(query, 'filters', None)
        data, granularity = await data_access.fetch_queue_flow_rates(
            queue_name, start_time, end_time, filters
        )
        
        return {"data": data, "granularity": granularity}
    
    @staticmethod
    async def get_global_stats(data_access) -> Dict[str, Any]:
        stats_data = await data_access.fetch_global_stats()
        return {
            "success": True,
            "data": stats_data
        }
    
    @staticmethod
    async def get_queues_detail(data_access) -> Dict[str, Any]:
        queues_data = await data_access.fetch_queues_data()
        return {
            "success": True,
            "data": queues_data
        }
    
    @staticmethod
    async def delete_queue(queue_name: str) -> Dict[str, Any]:
        logger.info(f"删除队列请求: {queue_name}")
        return {
            "success": True,
            "message": f"队列 {queue_name} 已删除"
        }
    
    @staticmethod
    async def trim_queue(queue_name: str, max_length: int) -> Dict[str, Any]:
        logger.info(f"裁剪队列请求: {queue_name}, 保留 {max_length} 条消息")
        return {
            "success": True,
            "message": f"队列 {queue_name} 已裁剪至 {max_length} 条消息"
        }
    
    @staticmethod
    async def get_queue_stats_v2(
        namespace_data_access,
        namespace: str,
        queue: Optional[str] = None,
        start_time: Optional[datetime] = None,
        end_time: Optional[datetime] = None,
        time_range: Optional[str] = None
    ) -> Dict[str, Any]:
        conn = await namespace_data_access.manager.get_connection(namespace)
        
        redis_client = await conn.get_redis_client(decode=False)
        
        pg_session = None
        if conn.AsyncSessionLocal:
            pg_session = conn.AsyncSessionLocal()
        
        try:
            from jettask.webui.services.queue_stats_v2 import QueueStatsV2
            
            stats_service = QueueStatsV2(
                redis_client=redis_client,
                pg_session=pg_session,
                redis_prefix=conn.redis_prefix
            )
            
            time_filter = None
            if time_range or start_time or end_time:
                time_filter = {}
                
                if time_range and time_range != 'custom':
                    now = datetime.now(timezone.utc)
                    if time_range.endswith('m'):
                        minutes = int(time_range[:-1])
                        time_filter['start_time'] = now - timedelta(minutes=minutes)
                        time_filter['end_time'] = now
                    elif time_range.endswith('h'):
                        hours = int(time_range[:-1])
                        time_filter['start_time'] = now - timedelta(hours=hours)
                        time_filter['end_time'] = now
                    elif time_range.endswith('d'):
                        days = int(time_range[:-1])
                        time_filter['start_time'] = now - timedelta(days=days)
                        time_filter['end_time'] = now
                else:
                    if start_time:
                        time_filter['start_time'] = start_time
                    if end_time:
                        time_filter['end_time'] = end_time
            
            stats = await stats_service.get_queue_stats_grouped(time_filter)
            
            if queue:
                stats = [s for s in stats if s['queue_name'] == queue]
            
            return {
                "success": True,
                "data": stats
            }
            
        finally:
            if pg_session:
                await pg_session.close()
            await redis_client.aclose()
    
    @staticmethod
    async def get_tasks_v2(namespace_data_access, namespace: str, body: Dict[str, Any]) -> Dict[str, Any]:
        from sqlalchemy import text
        from datetime import datetime, timezone, timedelta
        
        queue_name = body.get('queue_name')
        page = body.get('page', 1)
        page_size = body.get('page_size', 20)
        filters = body.get('filters', [])
        time_range = body.get('time_range', '1h')
        start_time = body.get('start_time')
        end_time = body.get('end_time')
        sort_field = body.get('sort_field', 'created_at')
        sort_order = body.get('sort_order', 'desc')
        
        if not queue_name:
            raise ValueError("queue_name is required")
        
        conn = await namespace_data_access.manager.get_connection(namespace)
        
        if not conn.pg_config or not conn.async_engine:
            return {
                "success": True,
                "data": [],
                "total": 0
            }
        
        if start_time and end_time:
            start_dt = datetime.fromisoformat(start_time.replace('Z', '+00:00'))
            end_dt = datetime.fromisoformat(end_time.replace('Z', '+00:00'))
        else:
            end_dt = datetime.now(timezone.utc)
            time_deltas = {
                '15m': timedelta(minutes=15),
                '30m': timedelta(minutes=30),
                '1h': timedelta(hours=1),
                '3h': timedelta(hours=3),
                '6h': timedelta(hours=6),
                '12h': timedelta(hours=12),
                '1d': timedelta(days=1),
                '3d': timedelta(days=3),
                '7d': timedelta(days=7),
                '30d': timedelta(days=30)
            }
            delta = time_deltas.get(time_range, timedelta(hours=1))
            start_dt = end_dt - delta
        
        offset = (page - 1) * page_size
        
        async with conn.async_engine.begin() as pg_conn:
            conditions = [
                "t.namespace = :namespace",
                "t.queue = :queue",
                "t.created_at >= :start_time",
                "t.created_at <= :end_time"
            ]
            query_params = {
                "namespace": namespace,
                "queue": queue_name,
                "start_time": start_dt,
                "end_time": end_dt,
                "limit": page_size,
                "offset": offset
            }
            
            for i, filter_item in enumerate(filters):
                field = filter_item.get('field')
                operator = filter_item.get('operator')
                value = filter_item.get('value')
                
                if field and operator and value is not None:
                    param_key = f"filter_{i}"
                    
                    db_field_map = {
                        'id': 't.stream_id',
                        'task_name': "t.payload::jsonb->'event_data'->>'__task_name'",
                        'status': "t.payload::jsonb->>'status'",
                        'worker_id': "t.payload::jsonb->>'worker_id'",
                        'scheduled_task_id': 't.scheduled_task_id'
                    }
                    
                    db_field = db_field_map.get(field, f't.{field}')
                    
                    if operator == 'eq':
                        conditions.append(f"{db_field} = :{param_key}")
                        query_params[param_key] = value
                    elif operator == 'contains':
                        conditions.append(f"{db_field} LIKE :{param_key}")
                        query_params[param_key] = f"%{value}%"
                    elif operator == 'gt':
                        conditions.append(f"{db_field} > :{param_key}")
                        query_params[param_key] = value
                    elif operator == 'lt':
                        conditions.append(f"{db_field} < :{param_key}")
                        query_params[param_key] = value
            
            where_clause = " AND ".join(conditions)
            
            count_query = f"""
                SELECT COUNT(*) as total 
                FROM tasks t
                WHERE {where_clause}
            """
            count_result = await pg_conn.execute(text(count_query), query_params)
            total = count_result.fetchone().total
            
            sort_map = {
                'created_at': 't.created_at',
                'started_at': 't.started_at',
                'completed_at': 't.completed_at'
            }
            order_by = sort_map.get(sort_field, 't.created_at')
            order_direction = 'DESC' if sort_order == 'desc' else 'ASC'
            
            query = f"""
                SELECT 
                    t.stream_id as id,
                    t.payload::jsonb->'event_data'->>'__task_name' as task_name,
                    t.queue,
                    t.payload::jsonb->>'status' as status,
                    t.priority,
                    COALESCE((t.payload::jsonb->>'retry_count')::int, 0) as retry_count,
                    COALESCE((t.payload::jsonb->>'max_retry')::int, 3) as max_retry,
                    t.created_at,
                    (t.payload::jsonb->>'started_at')::timestamptz as started_at,
                    (t.payload::jsonb->>'completed_at')::timestamptz as completed_at,
                    t.payload::jsonb->>'worker_id' as worker_id,
                    t.payload::jsonb->>'error_message' as error_message,
                    (t.payload::jsonb->>'execution_time')::float as execution_time,
                    CASE 
                        WHEN t.payload::jsonb->>'completed_at' IS NOT NULL AND t.created_at IS NOT NULL 
                        THEN EXTRACT(EPOCH FROM ((t.payload::jsonb->>'completed_at')::timestamptz - t.created_at))
                        ELSE NULL 
                    END as duration,
                    t.scheduled_task_id,
                    t.source,
                    t.metadata
                FROM tasks t
                WHERE {where_clause}
                ORDER BY {order_by} {order_direction}
                LIMIT :limit OFFSET :offset
            """
            
            result = await pg_conn.execute(text(query), query_params)
            
            tasks = []
            for row in result:
                tasks.append({
                    "id": row.id,
                    "task_name": row.task_name or "unknown",
                    "queue": row.queue,
                    "status": row.status,
                    "priority": row.priority,
                    "retry_count": row.retry_count,
                    "max_retry": row.max_retry,
                    "created_at": row.created_at.isoformat() if row.created_at else None,
                    "started_at": row.started_at.isoformat() if row.started_at else None,
                    "completed_at": row.completed_at.isoformat() if row.completed_at else None,
                    "duration": round(row.duration, 2) if row.duration else None,
                    "execution_time": float(row.execution_time) if row.execution_time else None,
                    "worker_id": row.worker_id,
                    "error_message": row.error_message
                })
            
            return {
                "success": True,
                "data": tasks,
                "total": total
            }
    
    @staticmethod
    async def get_consumer_group_stats(namespace_data_access, namespace: str, group_name: str) -> Dict[str, Any]:
        conn = await namespace_data_access.manager.get_connection(namespace)
        
        if not conn.AsyncSessionLocal:
            raise ValueError("PostgreSQL not configured for this namespace")
        
        async with conn.AsyncSessionLocal() as session:
            query = text("""
                WITH group_stats AS (
                    SELECT 
                        tr.consumer_group,
                        tr.task_name,
                        COUNT(*) as total_tasks,
                        COUNT(CASE WHEN tr.status = 'success' THEN 1 END) as success_count,
                        COUNT(CASE WHEN tr.status = 'failed' THEN 1 END) as failed_count,
                        COUNT(CASE WHEN tr.status = 'running' THEN 1 END) as running_count,
                        AVG(tr.execution_time) as avg_execution_time,
                        MIN(tr.execution_time) as min_execution_time,
                        MAX(tr.execution_time) as max_execution_time,
                        AVG(tr.duration) as avg_duration,
                        MIN(tr.started_at) as first_task_time,
                        MAX(tr.completed_at) as last_task_time
                    FROM task_runs tr
                    WHERE tr.consumer_group = :group_name
                        AND tr.started_at > NOW() - INTERVAL '24 hours'
                    GROUP BY tr.consumer_group, tr.task_name
                ),
                hourly_stats AS (
                    SELECT 
                        DATE_TRUNC('hour', tr.started_at) as hour,
                        COUNT(*) as task_count,
                        AVG(tr.execution_time) as avg_exec_time
                    FROM task_runs tr
                    WHERE tr.consumer_group = :group_name
                        AND tr.started_at > NOW() - INTERVAL '24 hours'
                    GROUP BY DATE_TRUNC('hour', tr.started_at)
                    ORDER BY hour
                )
                SELECT 
                    (SELECT row_to_json(gs) FROM group_stats gs) as summary,
                    (SELECT json_agg(hs) FROM hourly_stats hs) as hourly_trend
            """)
            
            result = await session.execute(query, {'group_name': group_name})
            row = result.fetchone()
            
            if not row or not row.summary:
                return {
                    "success": True,
                    "data": {
                        "group_name": group_name,
                        "summary": {},
                        "hourly_trend": []
                    }
                }
            
            return {
                "success": True,
                "data": {
                    "group_name": group_name,
                    "summary": row.summary,
                    "hourly_trend": row.hourly_trend or []
                }
            }
    

    @classmethod
    async def get_queue_tasks_detail(
        cls,
        namespace: str,
        queue_name: str,
        pg_session: AsyncSession,
        start_time: Optional[datetime] = None,
        end_time: Optional[datetime] = None,
        time_range: Optional[str] = None
    ) -> Dict[str, Any]:
        try:
            time_range_result = cls._resolve_time_range(
                start_time=start_time,
                end_time=end_time,
                time_range=time_range,
                default_range="15m"
            )
            start_time = time_range_result.start_time
            end_time = time_range_result.end_time

            logger.info(f"获取队列 tasks 详情: namespace={namespace}, queue={queue_name}, time_range={start_time} to {end_time}")

            task_stats_stmt = select(
                TaskRun.task_name,
                func.count().filter(TaskRun.status == 'success').label('success_count'),
                func.count().filter(TaskRun.status == 'failed').label('failed_count'),
                func.avg(TaskRun.duration).label('avg_duration')
            ).select_from(TaskRun).join(
                Task, TaskRun.stream_id == Task.stream_id
            ).where(
                and_(
                    Task.queue == queue_name,
                    Task.namespace == namespace,
                    TaskRun.created_at >= start_time,
                    TaskRun.created_at <= end_time,
                    TaskRun.task_name.isnot(None)
                )
            ).group_by(TaskRun.task_name)

            result = await pg_session.execute(task_stats_stmt)
            rows = result.all()

            tasks = []
            for row in rows:
                total_count = row.success_count + row.failed_count
                success_rate = round((row.success_count / total_count) * 100, 2) if total_count > 0 else 0.0

                tasks.append({
                    "task_name": row.task_name,
                    "success_count": row.success_count or 0,
                    "failed_count": row.failed_count or 0,
                    "success_rate": success_rate,
                    "avg_duration": round(row.avg_duration, 4) if row.avg_duration else 0.0
                })

            return {
                "success": True,
                "queue_name": queue_name,
                "tasks": tasks,
                "total": len(tasks),
                "time_range": {
                    "start_time": start_time.isoformat() if start_time else None,
                    "end_time": end_time.isoformat() if end_time else None
                }
            }

        except Exception as e:
            logger.error(f"获取队列 tasks 详情失败: queue={queue_name}, error={e}", exc_info=True)
            return {
                "success": False,
                "error": str(e),
                "queue_name": queue_name,
                "tasks": []
            }

    @classmethod
    async def _query_from_aggregation_table(
        cls,
        pg_session: AsyncSession,
        namespace: str,
        queues: List[str],
        start_time: datetime,
        end_time: datetime,
        interval_seconds: int
    ) -> List[Dict[str, Any]]:
        stmt = select(
            TaskMetricsMinute.queue,
            TaskMetricsMinute.time_bucket,
            TaskMetricsMinute.task_count
        ).where(
            and_(
                TaskMetricsMinute.namespace == namespace,
                TaskMetricsMinute.queue.in_(queues),
                TaskMetricsMinute.time_bucket >= start_time,
                TaskMetricsMinute.time_bucket <= end_time
            )
        ).order_by(
            TaskMetricsMinute.queue,
            TaskMetricsMinute.time_bucket
        )

        result = await pg_session.execute(stmt)
        rows = result.fetchall()

        time_buckets = []
        if interval_seconds > 60:
            start_timestamp = int(start_time.timestamp() / interval_seconds) * interval_seconds
            current = datetime.fromtimestamp(start_timestamp, tz=timezone.utc)
            end_timestamp = int(end_time.timestamp() / interval_seconds) * interval_seconds
            end_aligned = datetime.fromtimestamp(end_timestamp, tz=timezone.utc)
        else:
            current = start_time.replace(second=0, microsecond=0)
            end_aligned = end_time.replace(second=0, microsecond=0)

        while current <= end_aligned:
            time_buckets.append(current)
            current += timedelta(seconds=interval_seconds)

        queue_data_map = {}
        for queue_name in queues:
            queue_data_map[queue_name] = {bucket: 0 for bucket in time_buckets}

        for row in rows:
            queue_name = row.queue
            time_bucket = row.time_bucket
            count = row.task_count

            if interval_seconds > 60:
                bucket_timestamp = int(time_bucket.timestamp() / interval_seconds) * interval_seconds
                aligned_bucket = datetime.fromtimestamp(bucket_timestamp, tz=timezone.utc)

                if queue_name in queue_data_map and aligned_bucket in queue_data_map[queue_name]:
                    queue_data_map[queue_name][aligned_bucket] += count
            else:
                if queue_name in queue_data_map and time_bucket in queue_data_map[queue_name]:
                    queue_data_map[queue_name][time_bucket] = count

        data = []
        for queue_name in queues:
            trend = []
            for bucket in sorted(time_buckets):
                trend.append({
                    "time": bucket.isoformat(),
                    "count": queue_data_map[queue_name].get(bucket, 0)
                })

            data.append({
                "queue_name": queue_name,
                "trend": trend
            })

        return data

    @classmethod
    async def _query_from_tasks_table(
        cls,
        pg_session: AsyncSession,
        namespace: str,
        queues: List[str],
        start_time: datetime,
        end_time: datetime,
        interval_seconds: int
    ) -> List[Dict[str, Any]]:
        stmt = select(
            Task.queue,
            Task.created_at
        ).where(
            and_(
                Task.namespace == namespace,
                Task.queue.in_(queues),
                Task.created_at >= start_time,
                Task.created_at <= end_time
            )
        ).order_by(Task.created_at)

        result = await pg_session.execute(stmt)
        tasks = result.fetchall()

        time_buckets = []
        current = start_time
        while current <= end_time:
            time_buckets.append(current)
            current += timedelta(seconds=interval_seconds)

        queue_data_map = {}
        for queue_name in queues:
            queue_data_map[queue_name] = {bucket: 0 for bucket in time_buckets}

        for row in tasks:
            queue_name = row.queue
            created_at = row.created_at

            bucket_timestamp = int(created_at.timestamp() / interval_seconds) * interval_seconds
            bucket = datetime.fromtimestamp(bucket_timestamp, tz=timezone.utc)

            if queue_name in queue_data_map and bucket in queue_data_map[queue_name]:
                queue_data_map[queue_name][bucket] += 1

        data = []
        for queue_name in queues:
            trend = []
            for bucket in sorted(time_buckets):
                trend.append({
                    "time": bucket.isoformat(),
                    "count": queue_data_map[queue_name].get(bucket, 0)
                })

            data.append({
                "queue_name": queue_name,
                "trend": trend
            })

        return data


    @classmethod
    def _validate_where_clause(cls, where_clause: str) -> None:
        forbidden_keywords = [
            'DROP', 'DELETE', 'UPDATE', 'INSERT', 'TRUNCATE', 'ALTER', 'CREATE',
            'GRANT', 'REVOKE', 'EXEC', 'EXECUTE', 'UNION', 'INTO',
            'GROUP BY', 'ORDER BY', 'HAVING', 'LIMIT', 'OFFSET',
            '--', '/*', '*/', ';'
        ]
        upper_clause = where_clause.upper()
        for keyword in forbidden_keywords:
            if keyword in upper_clause:
                raise ValueError(f"WHERE 子句包含不允许的关键字: {keyword}")

    @classmethod
    def _resolve_ambiguous_columns(cls, where_clause: str) -> str:
        import re

        ambiguous_columns = {
            'stream_id': 'tasks',
            'created_at': 'tasks',  
            'namespace': 'tasks',   
        }

        result = where_clause

        for col, default_table in ambiguous_columns.items():
            pattern = rf'(?<![.\w])\b{col}\b(?!\s*\.)'

            if re.search(pattern, result, re.IGNORECASE):
                result = re.sub(
                    pattern,
                    f'{default_table}.{col}',
                    result,
                    flags=re.IGNORECASE
                )
                logger.debug(f"自动补全列名前缀: {col} -> {default_table}.{col}")

        return result

    @classmethod
    async def _build_task_runs_query_conditions(
        cls,
        namespace: str,
        registry,
        queue: str,
        start_time: Optional[datetime] = None,
        end_time: Optional[datetime] = None,
        time_range: Optional[str] = None,
        status: Optional[List[str]] = None,
        task_names: Optional[List[str]] = None,
        where_clause: Optional[str] = None
    ) -> QueryConditionsResult:
        from sqlalchemy import text

        time_range_result = cls._resolve_time_range(
            start_time=start_time,
            end_time=end_time,
            time_range=time_range,
            default_range="1h"
        )

        queue_result = await cls._resolve_queues([queue], registry, expand_priority=True)
        all_priority_queues = queue_result.all_priority_queues

        if where_clause and where_clause.strip():
            cls._validate_where_clause(where_clause)

        conditions = [
            Task.namespace == namespace,
            Task.queue.in_(all_priority_queues),
            Task.created_at >= time_range_result.start_time,
            Task.created_at <= time_range_result.end_time
        ]

        if status:
            conditions.append(TaskRun.status.in_(status))

        if task_names:
            conditions.append(TaskRun.task_name.in_(task_names))

        if where_clause and where_clause.strip():
            resolved_clause = cls._resolve_ambiguous_columns(where_clause.strip())
            conditions.append(text(resolved_clause))

        logger.info(
            f"构建任务查询条件: namespace={namespace}, queue={queue}, "
            f"priority_queues={all_priority_queues}, "
            f"time_range={time_range_result.start_time} to {time_range_result.end_time}, "
            f"status={status}, task_names={task_names}, where_clause={where_clause}"
        )

        return QueryConditionsResult(
            conditions=conditions,
            all_priority_queues=all_priority_queues,
            time_range_result=time_range_result
        )

    @classmethod
    async def get_task_runs_list(
        cls,
        namespace: str,
        pg_session: AsyncSession,
        registry,
        queue: str,
        start_time: Optional[datetime] = None,
        end_time: Optional[datetime] = None,
        time_range: Optional[str] = None,
        status: Optional[List[str]] = None,
        task_names: Optional[List[str]] = None,
        where_clause: Optional[str] = None,
        limit: int = 50,
        cursor: Optional[str] = None,
        sort_field: str = "created_at",
        sort_order: str = "desc",
        include_fields: Optional[List[str]] = None
    ) -> Dict[str, Any]:
        from sqlalchemy import desc, asc

        try:
            query_result = await cls._build_task_runs_query_conditions(
                namespace=namespace,
                registry=registry,
                queue=queue,
                start_time=start_time,
                end_time=end_time,
                time_range=time_range,
                status=status,
                task_names=task_names,
                where_clause=where_clause
            )

            conditions = query_result.conditions
            time_range_result = query_result.time_range_result

            if not query_result.all_priority_queues:
                return {
                    "success": True,
                    "data": [],
                    "queue": queue,
                    "time_range": {
                        "start_time": time_range_result.start_time.isoformat(),
                        "end_time": time_range_result.end_time.isoformat()
                    },
                    "pagination": {
                        "next_cursor": None,
                        "has_more": False
                    }
                }

            cursor_data = None
            if cursor:
                try:
                    cursor_data = json.loads(base64.b64decode(cursor).decode('utf-8'))
                except Exception:
                    raise ValueError("无效的 cursor")

            if cursor_data:
                cursor_stream_id = cursor_data.get("stream_id")
                if cursor_stream_id:
                    if sort_order.lower() == 'desc':
                        conditions.append(Task.stream_id < cursor_stream_id)
                    else:
                        conditions.append(Task.stream_id > cursor_stream_id)

            _large_field_map = {
                'payload': Task.payload,
                'result': TaskRun.result,
                'error': TaskRun.error,
                'metadata': Task.task_metadata,
            }
            _include = set(include_fields) if include_fields else set()

            select_columns = [
                Task.stream_id,
                Task.scheduled_task_id,
                Task.priority,
                Task.created_at,
                Task.trigger_time,
                Task.source,
                TaskRun.task_name,
                TaskRun.status,
                TaskRun.started_at,
                TaskRun.completed_at,
                TaskRun.retries,
                TaskRun.duration,
                TaskRun.consumer,
            ]
            for field_name in ('payload', 'result', 'error', 'metadata'):
                if field_name in _include:
                    select_columns.append(_large_field_map[field_name].label(field_name))

            query_stmt = select(*select_columns).select_from(
                Task
            ).outerjoin(
                TaskRun, Task.stream_id == TaskRun.stream_id
            ).where(and_(*conditions))

            sort_column = None
            if sort_field in ['stream_id', 'scheduled_task_id', 'priority', 'created_at', 'trigger_time', 'source']:
                sort_column = getattr(Task, sort_field, None)
            elif sort_field in ['task_name', 'status', 'started_at', 'completed_at', 'retries', 'duration', 'consumer']:
                sort_column = getattr(TaskRun, sort_field, None)

            if sort_column is not None:
                if sort_order.lower() == 'desc':
                    query_stmt = query_stmt.order_by(desc(sort_column))
                else:
                    query_stmt = query_stmt.order_by(asc(sort_column))
            else:
                query_stmt = query_stmt.order_by(desc(Task.created_at))

            query_stmt = query_stmt.limit(limit + 1)

            result = await pg_session.execute(query_stmt)
            rows = result.fetchall()

            has_more = len(rows) > limit
            if has_more:
                rows = rows[:limit]

            data = []
            last_stream_id = None
            for row in rows:
                delay = None
                if row.started_at and row.trigger_time:
                    delay = (row.started_at - row.trigger_time).total_seconds()

                record = {
                    "stream_id": row.stream_id,
                    "task_name": row.task_name,
                    "status": row.status,
                    "priority": row.priority,
                    "retries": row.retries or 0,
                    "created_at": row.created_at.isoformat() if row.created_at else None,
                    "started_at": row.started_at.isoformat() if row.started_at else None,
                    "completed_at": row.completed_at.isoformat() if row.completed_at else None,
                    "duration": float(row.duration) if row.duration else None,
                    "delay": round(delay, 3) if delay is not None else None,
                    "consumer": row.consumer,
                    "source": row.source,
                    "scheduled_task_id": row.scheduled_task_id
                }
                for field_name in ('payload', 'result', 'error', 'metadata'):
                    if field_name in _include:
                        record[field_name] = getattr(row, field_name, None)
                data.append(record)
                last_stream_id = row.stream_id

            next_cursor = None
            if has_more and last_stream_id:
                cursor_obj = {"stream_id": last_stream_id}
                next_cursor = base64.b64encode(json.dumps(cursor_obj).encode('utf-8')).decode('utf-8')

            result = {
                "success": True,
                "data": data,
                "queue": queue,
                "time_range": {
                    "start_time": time_range_result.start_time.isoformat(),
                    "end_time": time_range_result.end_time.isoformat()
                },
                "pagination": {
                    "next_cursor": next_cursor,
                    "has_more": has_more
                }
            }

            if not cursor:
                try:
                    count_stmt = select(func.count(Task.stream_id)).select_from(
                        Task
                    ).outerjoin(
                        TaskRun, Task.stream_id == TaskRun.stream_id
                    ).where(and_(*conditions))

                    count_result = await pg_session.execute(count_stmt)
                    estimated_total = count_result.scalar() or 0
                    result["pagination"]["estimated_total"] = estimated_total
                except Exception as count_err:
                    logger.warning(f"获取预估总数失败: {count_err}")

            return result

        except ValueError as e:
            logger.warning(f"参数错误: {e}")
            raise
        except Exception as e:
            logger.error(f"获取任务执行记录失败: {e}", exc_info=True)
            return {
                "success": False,
                "error": str(e),
                "data": [],
                "queue": queue,
                "pagination": {
                    "next_cursor": None,
                    "has_more": False
                }
            }

    TREND_GROUP_BY_COLUMNS = {
        'priority': Task.priority,
        'source': Task.source,
        'task_name': TaskRun.task_name,
        'consumer': TaskRun.consumer,
        'status': TaskRun.status,
    }

    @classmethod
    async def get_task_runs_trend(
        cls,
        namespace: str,
        pg_session: AsyncSession,
        registry,
        queue: str,
        start_time: Optional[datetime] = None,
        end_time: Optional[datetime] = None,
        time_range: Optional[str] = None,
        status: Optional[List[str]] = None,
        task_names: Optional[List[str]] = None,
        where_clause: Optional[str] = None,
        group_by: Optional[List[str]] = None
    ) -> Dict[str, Any]:
        from sqlalchemy import literal_column

        try:
            group_by_columns = []
            group_by_names = []
            if group_by:
                for dim in group_by:
                    if dim not in cls.TREND_GROUP_BY_COLUMNS:
                        raise ValueError(
                            f"无效的聚合维度: {dim}。"
                            f"支持: {', '.join(cls.TREND_GROUP_BY_COLUMNS.keys())}"
                        )
                    group_by_columns.append(cls.TREND_GROUP_BY_COLUMNS[dim])
                    group_by_names.append(dim)

            query_result = await cls._build_task_runs_query_conditions(
                namespace=namespace,
                registry=registry,
                queue=queue,
                start_time=start_time,
                end_time=end_time,
                time_range=time_range,
                status=status,
                task_names=task_names,
                where_clause=where_clause
            )

            conditions = query_result.conditions
            time_range_result = query_result.time_range_result
            interval_seconds = time_range_result.interval_seconds

            if not query_result.all_priority_queues:
                return {
                    "success": True,
                    "data": {
                        "trend": [] if not group_by else {}
                    },
                    "queue": queue,
                    "group_by": group_by,
                    "granularity": time_range_result.granularity,
                    "time_range": {
                        "start_time": time_range_result.start_time.isoformat(),
                        "end_time": time_range_result.end_time.isoformat(),
                        "interval": time_range_result.interval,
                        "interval_seconds": interval_seconds
                    }
                }

            start_ts = time_range_result.start_time.timestamp()
            aligned_start_ts = (int(start_ts) // interval_seconds) * interval_seconds
            current = datetime.fromtimestamp(aligned_start_ts, tz=timezone.utc)

            time_buckets = []
            while current <= time_range_result.end_time:
                time_buckets.append(current)
                current += timedelta(seconds=interval_seconds)

            trunc_unit = 'minute' if interval_seconds < 3600 else 'hour'
            time_bucket_expr = func.date_trunc(trunc_unit, TaskRun.started_at)

            if group_by_columns:
                select_columns = [time_bucket_expr.label('time_bucket')]
                for i, col in enumerate(group_by_columns):
                    select_columns.append(col.label(f'group_{i}'))
                select_columns.extend([
                    func.count().label('total_count'),
                    func.count().filter(TaskRun.status == 'success').label('success_count'),
                    func.count().filter(TaskRun.status == 'failed').label('failed_count')
                ])

                group_by_exprs = [time_bucket_expr] + group_by_columns

                query_stmt = select(*select_columns).select_from(
                    Task
                ).outerjoin(
                    TaskRun, Task.stream_id == TaskRun.stream_id
                ).where(and_(*conditions)).group_by(
                    *group_by_exprs
                ).order_by(literal_column('time_bucket'))
            else:
                query_stmt = select(
                    time_bucket_expr.label('time_bucket'),
                    func.count().label('total_count'),
                    func.count().filter(TaskRun.status == 'success').label('success_count'),
                    func.count().filter(TaskRun.status == 'failed').label('failed_count')
                ).select_from(
                    Task
                ).outerjoin(
                    TaskRun, Task.stream_id == TaskRun.stream_id
                ).where(and_(*conditions)).group_by(
                    time_bucket_expr
                ).order_by(literal_column('time_bucket'))

            result = await pg_session.execute(query_stmt)
            rows = result.fetchall()

            if group_by_columns:
                grouped_data: Dict[str, Dict[datetime, Dict]] = {}

                for row in rows:
                    if row.time_bucket:
                        bucket_timestamp = int(row.time_bucket.timestamp() / interval_seconds) * interval_seconds
                        aligned_bucket = datetime.fromtimestamp(bucket_timestamp, tz=timezone.utc)

                        group_values = []
                        for i in range(len(group_by_columns)):
                            val = getattr(row, f'group_{i}', None)
                            group_values.append(str(val) if val is not None else "(unknown)")
                        group_key = "|".join(group_values)

                        if group_key not in grouped_data:
                            grouped_data[group_key] = {}

                        if aligned_bucket not in grouped_data[group_key]:
                            grouped_data[group_key][aligned_bucket] = {
                                "total_count": 0,
                                "success_count": 0,
                                "failed_count": 0
                            }

                        grouped_data[group_key][aligned_bucket]["total_count"] += row.total_count
                        grouped_data[group_key][aligned_bucket]["success_count"] += row.success_count
                        grouped_data[group_key][aligned_bucket]["failed_count"] += row.failed_count

                trend_by_group = {}
                for group_key, bucket_data in grouped_data.items():
                    group_values = group_key.split("|")
                    dimension_values = {}
                    for i, name in enumerate(group_by_names):
                        dimension_values[name] = group_values[i] if i < len(group_values) else "(unknown)"

                    trend = []
                    for bucket in time_buckets:
                        data_point = bucket_data.get(bucket, {
                            "total_count": 0,
                            "success_count": 0,
                            "failed_count": 0
                        })
                        trend.append({
                            "time": bucket.isoformat(),
                            "total_count": data_point["total_count"],
                            "success_count": data_point["success_count"],
                            "failed_count": data_point["failed_count"]
                        })

                    trend_by_group[group_key] = {
                        "dimensions": dimension_values,
                        "trend": trend
                    }

                return {
                    "success": True,
                    "data": trend_by_group,
                    "queue": queue,
                    "group_by": group_by,
                    "granularity": time_range_result.granularity,
                    "time_range": {
                        "start_time": time_range_result.start_time.isoformat(),
                        "end_time": time_range_result.end_time.isoformat(),
                        "interval": time_range_result.interval,
                        "interval_seconds": interval_seconds
                    }
                }
            else:
                bucket_data = {}
                for row in rows:
                    if row.time_bucket:
                        bucket_timestamp = int(row.time_bucket.timestamp() / interval_seconds) * interval_seconds
                        aligned_bucket = datetime.fromtimestamp(bucket_timestamp, tz=timezone.utc)

                        if aligned_bucket not in bucket_data:
                            bucket_data[aligned_bucket] = {
                                "total_count": 0,
                                "success_count": 0,
                                "failed_count": 0
                            }
                        bucket_data[aligned_bucket]["total_count"] += row.total_count
                        bucket_data[aligned_bucket]["success_count"] += row.success_count
                        bucket_data[aligned_bucket]["failed_count"] += row.failed_count

                trend = []
                for bucket in time_buckets:
                    data_point = bucket_data.get(bucket, {
                        "total_count": 0,
                        "success_count": 0,
                        "failed_count": 0
                    })
                    trend.append({
                        "time": bucket.isoformat(),
                        "total_count": data_point["total_count"],
                        "success_count": data_point["success_count"],
                        "failed_count": data_point["failed_count"]
                    })

                return {
                    "success": True,
                    "data": {
                        "trend": trend
                    },
                    "queue": queue,
                    "group_by": None,
                    "granularity": time_range_result.granularity,
                    "time_range": {
                        "start_time": time_range_result.start_time.isoformat(),
                        "end_time": time_range_result.end_time.isoformat(),
                        "interval": time_range_result.interval,
                        "interval_seconds": interval_seconds
                    }
                }

        except ValueError as e:
            logger.warning(f"参数错误: {e}")
            raise
        except Exception as e:
            logger.error(f"获取任务执行趋势失败: {e}", exc_info=True)
            return {
                "success": False,
                "error": str(e),
                "data": {"trend": []} if not group_by else {},
                "queue": queue
            }

    @classmethod
    async def get_task_run_detail(
        cls,
        namespace: str,
        pg_session: AsyncSession,
        stream_id: str
    ) -> Dict[str, Any]:
        try:
            query_stmt = select(
                Task.stream_id,
                Task.queue,
                Task.namespace,
                Task.scheduled_task_id,
                Task.payload,
                Task.priority,
                Task.delay,
                Task.created_at,
                Task.trigger_time,
                Task.source,
                Task.task_metadata,
                TaskRun.task_name,
                TaskRun.status,
                TaskRun.result,
                TaskRun.error,
                TaskRun.started_at,
                TaskRun.completed_at,
                TaskRun.retries,
                TaskRun.duration,
                TaskRun.consumer
            ).select_from(
                Task
            ).outerjoin(
                TaskRun, Task.stream_id == TaskRun.stream_id
            ).where(
                and_(
                    Task.namespace == namespace,
                    Task.stream_id == stream_id
                )
            )

            result = await pg_session.execute(query_stmt)
            row = result.fetchone()

            if not row:
                return {
                    "success": False,
                    "error": f"任务不存在: {stream_id}",
                    "data": None
                }

            delay_seconds = None
            if row.started_at and row.trigger_time:
                delay_seconds = (row.started_at - row.trigger_time).total_seconds()

            return {
                "success": True,
                "data": {
                    "stream_id": row.stream_id,
                    "queue": row.queue,
                    "namespace": row.namespace,
                    "scheduled_task_id": row.scheduled_task_id,
                    "payload": row.payload,
                    "priority": row.priority,
                    "delay": row.delay,
                    "created_at": row.created_at.isoformat() if row.created_at else None,
                    "trigger_time": row.trigger_time.isoformat() if row.trigger_time else None,
                    "source": row.source,
                    "metadata": row.task_metadata,
                    "task_name": row.task_name,
                    "status": row.status,
                    "result": row.result,
                    "error": row.error,
                    "started_at": row.started_at.isoformat() if row.started_at else None,
                    "completed_at": row.completed_at.isoformat() if row.completed_at else None,
                    "retries": row.retries or 0,
                    "duration": float(row.duration) if row.duration else None,
                    "delay_seconds": round(delay_seconds, 3) if delay_seconds is not None else None,
                    "consumer": row.consumer
                }
            }

        except Exception as e:
            logger.error(f"获取任务详情失败: stream_id={stream_id}, error={e}", exc_info=True)
            return {
                "success": False,
                "error": str(e),
                "data": None
            }


    @classmethod
    async def get_backlog_trend(
        cls,
        namespace: str,
        pg_session: AsyncSession,
        queues: List[str],
        task_names: Optional[List[str]] = None,
        start_time: Optional[datetime] = None,
        end_time: Optional[datetime] = None,
        time_range: Optional[str] = None,
        aggregate_by_queue: bool = False
    ) -> Dict[str, Any]:
        try:
            time_range_result = cls._resolve_time_range(
                start_time=start_time,
                end_time=end_time,
                time_range=time_range,
                default_range="1h",
                min_interval_seconds=60
            )

            interval_seconds = time_range_result.interval_seconds

            conditions = [
                QueueSnapshot.namespace == namespace,
                QueueSnapshot.queue.in_(queues),
                QueueSnapshot.time_bucket >= time_range_result.start_time,
                QueueSnapshot.time_bucket <= time_range_result.end_time
            ]

            conditions.append(QueueSnapshot.task_name != '')

            if aggregate_by_queue:
                pass
            elif task_names:
                conditions.append(QueueSnapshot.task_name.in_(task_names))

            stmt = select(
                QueueSnapshot.time_bucket,
                QueueSnapshot.queue,
                QueueSnapshot.task_name,
                QueueSnapshot.backlog,
                QueueSnapshot.pending_read,
                QueueSnapshot.send_offset,
                QueueSnapshot.ack_offset,
                QueueSnapshot.read_offset
            ).where(
                and_(*conditions)
            ).order_by(
                QueueSnapshot.queue,
                QueueSnapshot.task_name,
                QueueSnapshot.time_bucket
            )

            result = await pg_session.execute(stmt)
            rows = result.fetchall()

            time_buckets = cls._generate_backlog_time_buckets(
                time_range_result.start_time,
                time_range_result.end_time,
                interval_seconds
            )

            if aggregate_by_queue:
                data = cls._aggregate_backlog_by_queue(rows, queues, time_buckets, interval_seconds)
            else:
                data = cls._aggregate_backlog_by_task(rows, queues, time_buckets, interval_seconds)

            return {
                "success": True,
                "data": data,
                "granularity": time_range_result.granularity,
                "time_range": {
                    "start_time": time_range_result.start_time.isoformat(),
                    "end_time": time_range_result.end_time.isoformat(),
                    "interval": time_range_result.interval,
                    "interval_seconds": interval_seconds
                }
            }

        except Exception as e:
            logger.error(f"获取积压趋势失败: {e}", exc_info=True)
            return {
                "success": False,
                "error": str(e),
                "data": []
            }

    @classmethod
    async def get_backlog_summary(
        cls,
        namespace: str,
        pg_session: AsyncSession,
        queues: List[str],
        task_names: Optional[List[str]] = None
    ) -> Dict[str, Any]:
        try:
            latest_time_stmt = select(
                func.max(QueueSnapshot.time_bucket)
            ).where(
                and_(
                    QueueSnapshot.namespace == namespace,
                    QueueSnapshot.queue.in_(queues)
                )
            )
            latest_result = await pg_session.execute(latest_time_stmt)
            latest_time = latest_result.scalar()

            if not latest_time:
                return {
                    "success": True,
                    "data": [],
                    "latest_time": None
                }

            conditions = [
                QueueSnapshot.namespace == namespace,
                QueueSnapshot.queue.in_(queues),
                QueueSnapshot.time_bucket == latest_time
            ]

            conditions.append(QueueSnapshot.task_name != '')

            if task_names:
                conditions.append(QueueSnapshot.task_name.in_(task_names))

            stmt = select(
                QueueSnapshot.queue,
                QueueSnapshot.task_name,
                QueueSnapshot.backlog,
                QueueSnapshot.pending_read,
                QueueSnapshot.send_offset,
                QueueSnapshot.ack_offset,
                QueueSnapshot.read_offset
            ).where(
                and_(*conditions)
            ).order_by(
                QueueSnapshot.queue,
                QueueSnapshot.task_name
            )

            result = await pg_session.execute(stmt)
            rows = result.fetchall()

            data = []
            for row in rows:
                data.append({
                    "queue": row.queue,
                    "task_name": row.task_name,
                    "backlog": row.backlog,
                    "pending_read": row.pending_read,
                    "send_offset": row.send_offset,
                    "ack_offset": row.ack_offset,
                    "read_offset": row.read_offset
                })

            return {
                "success": True,
                "data": data,
                "latest_time": latest_time.isoformat() if latest_time else None
            }

        except Exception as e:
            logger.error(f"获取积压汇总失败: {e}", exc_info=True)
            return {
                "success": False,
                "error": str(e),
                "data": []
            }

    @classmethod
    def _generate_backlog_time_buckets(
        cls,
        start_time: datetime,
        end_time: datetime,
        interval_seconds: int
    ) -> List[datetime]:
        current = start_time.replace(second=0, microsecond=0)
        buckets = []

        while current <= end_time:
            buckets.append(current)
            current += timedelta(seconds=interval_seconds)

        return buckets

    @classmethod
    def _aggregate_backlog_by_queue(
        cls,
        rows,
        queues: List[str],
        time_buckets: List[datetime],
        interval_seconds: int
    ) -> List[Dict[str, Any]]:
        queue_data = {q: {} for q in queues}

        for row in rows:
            queue = row.queue
            time_bucket = row.time_bucket
            aligned_bucket = cls._align_backlog_time_bucket(time_bucket, interval_seconds)

            if queue in queue_data:
                queue_data[queue][aligned_bucket] = {
                    "backlog": row.backlog,
                    "pending_read": row.pending_read
                }

        result = []
        for queue in queues:
            trend = []
            for bucket in time_buckets:
                data_point = queue_data.get(queue, {}).get(bucket, {
                    "backlog": 0,
                    "pending_read": 0
                })
                trend.append({
                    "time": bucket.isoformat(),
                    "backlog": data_point.get("backlog", 0),
                    "pending_read": data_point.get("pending_read", 0)
                })

            result.append({
                "queue": queue,
                "trend": trend
            })

        return result

    @classmethod
    def _aggregate_backlog_by_task(
        cls,
        rows,
        queues: List[str],
        time_buckets: List[datetime],
        interval_seconds: int
    ) -> List[Dict[str, Any]]:
        queue_task_data: Dict[str, Dict[str, Dict[datetime, Dict]]] = {}

        for row in rows:
            queue = row.queue
            task_name = row.task_name
            time_bucket = row.time_bucket
            aligned_bucket = cls._align_backlog_time_bucket(time_bucket, interval_seconds)

            if queue not in queue_task_data:
                queue_task_data[queue] = {}
            if task_name not in queue_task_data[queue]:
                queue_task_data[queue][task_name] = {}

            queue_task_data[queue][task_name][aligned_bucket] = {
                "backlog": row.backlog,
                "pending_read": row.pending_read
            }

        result = []
        for queue in queues:
            if queue not in queue_task_data:
                continue

            tasks = []
            for task_name, bucket_data in queue_task_data[queue].items():
                trend = []
                for bucket in time_buckets:
                    data_point = bucket_data.get(bucket, {
                        "backlog": 0,
                        "pending_read": 0
                    })
                    trend.append({
                        "time": bucket.isoformat(),
                        "backlog": data_point.get("backlog", 0),
                        "pending_read": data_point.get("pending_read", 0)
                    })

                tasks.append({
                    "task_name": task_name,
                    "trend": trend
                })

            result.append({
                "queue": queue,
                "tasks": tasks
            })

        return result

    @classmethod
    def _align_backlog_time_bucket(cls, time_bucket: datetime, interval_seconds: int) -> datetime:
        if interval_seconds <= 60:
            return time_bucket.replace(second=0, microsecond=0)

        timestamp = int(time_bucket.timestamp())
        aligned_timestamp = (timestamp // interval_seconds) * interval_seconds
        return datetime.fromtimestamp(aligned_timestamp, tz=timezone.utc)


    @classmethod
    async def get_worker_trend(
        cls,
        namespace: str,
        pg_session: AsyncSession,
        start_time: Optional[datetime] = None,
        end_time: Optional[datetime] = None,
        time_range: Optional[str] = None
    ) -> Dict[str, Any]:
        try:
            time_range_result = cls._resolve_time_range(
                start_time=start_time,
                end_time=end_time,
                time_range=time_range,
                default_range="1h",
                min_interval_seconds=60
            )

            interval_seconds = time_range_result.interval_seconds

            conditions = [
                QueueSnapshot.namespace == namespace,
                QueueSnapshot.time_bucket >= time_range_result.start_time,
                QueueSnapshot.time_bucket <= time_range_result.end_time,
                QueueSnapshot.queue == '',
                QueueSnapshot.task_name == ''
            ]

            stmt = select(
                QueueSnapshot.time_bucket,
                QueueSnapshot.online_count,
                QueueSnapshot.offline_count,
                QueueSnapshot.total_count
            ).where(
                and_(*conditions)
            ).order_by(
                QueueSnapshot.time_bucket
            )

            result = await pg_session.execute(stmt)
            rows = result.fetchall()

            time_buckets = cls._generate_backlog_time_buckets(
                time_range_result.start_time,
                time_range_result.end_time,
                interval_seconds
            )

            bucket_data = {}
            for row in rows:
                time_bucket = row.time_bucket
                aligned_bucket = cls._align_backlog_time_bucket(time_bucket, interval_seconds)

                if aligned_bucket not in bucket_data:
                    bucket_data[aligned_bucket] = {
                        "online_count": 0,
                        "offline_count": 0,
                        "total_count": 0
                    }

                bucket_data[aligned_bucket] = {
                    "online_count": row.online_count,
                    "offline_count": row.offline_count,
                    "total_count": row.total_count
                }

            trend = []
            for bucket in time_buckets:
                data_point = bucket_data.get(bucket, {
                    "online_count": 0,
                    "offline_count": 0,
                    "total_count": 0
                })
                trend.append({
                    "time": bucket.isoformat(),
                    "online_count": data_point["online_count"],
                    "offline_count": data_point["offline_count"],
                    "total_count": data_point["total_count"]
                })

            return {
                "success": True,
                "data": {
                    "trend": trend
                },
                "granularity": time_range_result.granularity,
                "time_range": {
                    "start_time": time_range_result.start_time.isoformat(),
                    "end_time": time_range_result.end_time.isoformat(),
                    "interval": time_range_result.interval,
                    "interval_seconds": interval_seconds
                }
            }

        except Exception as e:
            logger.error(f"获取 Worker 趋势失败: {e}", exc_info=True)
            return {
                "success": False,
                "error": str(e),
                "data": {"trend": []}
            }

    @classmethod
    async def get_worker_trend_by_queue(
        cls,
        namespace: str,
        pg_session: AsyncSession,
        queue: str,
        start_time: Optional[datetime] = None,
        end_time: Optional[datetime] = None,
        time_range: Optional[str] = None
    ) -> Dict[str, Any]:
        try:
            time_range_result = cls._resolve_time_range(
                start_time=start_time,
                end_time=end_time,
                time_range=time_range,
                default_range="1h",
                min_interval_seconds=60
            )

            interval_seconds = time_range_result.interval_seconds

            conditions = [
                QueueSnapshot.namespace == namespace,
                QueueSnapshot.queue == queue,
                QueueSnapshot.task_name == '',
                QueueSnapshot.time_bucket >= time_range_result.start_time,
                QueueSnapshot.time_bucket <= time_range_result.end_time,
            ]

            stmt = select(
                QueueSnapshot.time_bucket,
                QueueSnapshot.online_count,
                QueueSnapshot.offline_count,
                QueueSnapshot.total_count
            ).where(
                and_(*conditions)
            ).order_by(
                QueueSnapshot.time_bucket
            )

            result = await pg_session.execute(stmt)
            rows = result.fetchall()

            time_buckets = cls._generate_backlog_time_buckets(
                time_range_result.start_time,
                time_range_result.end_time,
                interval_seconds
            )

            bucket_data = {}
            for row in rows:
                aligned_bucket = cls._align_backlog_time_bucket(row.time_bucket, interval_seconds)
                bucket_data[aligned_bucket] = {
                    "online_count": row.online_count,
                    "offline_count": row.offline_count,
                    "total_count": row.total_count
                }

            trend = []
            for bucket in time_buckets:
                data_point = bucket_data.get(bucket, {
                    "online_count": 0,
                    "offline_count": 0,
                    "total_count": 0
                })
                trend.append({
                    "time": bucket.isoformat(),
                    "online_count": data_point["online_count"],
                    "offline_count": data_point["offline_count"],
                    "total_count": data_point["total_count"]
                })

            return {
                "success": True,
                "data": {
                    "trend": trend
                },
                "queue": queue,
                "granularity": time_range_result.granularity,
                "time_range": {
                    "start_time": time_range_result.start_time.isoformat(),
                    "end_time": time_range_result.end_time.isoformat(),
                    "interval": time_range_result.interval,
                    "interval_seconds": interval_seconds
                }
            }

        except Exception as e:
            logger.error(f"获取队列级 Worker 趋势失败: queue={queue}, error={e}", exc_info=True)
            return {
                "success": False,
                "error": str(e),
                "data": {"trend": []}
            }



"""数据采集模块

定时采集队列快照数据（积压指标 + Worker 指标），写入 queue_snapshot 表。
"""

import asyncio
import json
import logging
import time
from collections import defaultdict
from datetime import datetime, timezone, timedelta
from typing import Dict, List, Optional, Set, TYPE_CHECKING

from sqlalchemy.dialects.postgresql import insert
from sqlalchemy import delete

from jettask.db.models.queue_snapshot import QueueSnapshot

if TYPE_CHECKING:
    from jettask import Jettask

logger = logging.getLogger(__name__)


class QueueSnapshotCollector:
    """队列快照采集器

    合并积压指标和 Worker 指标的采集，写入 queue_snapshot 表。

    特性：
    - 使用分布式锁，保证多进程环境下只有一个进程执行采集
    - 每分钟采集一次（可配置）
    - 记录任务级明细和 namespace 级汇总（Worker 全局去重）
    - 自动清理过期数据
    """

    LOCK_KEY_TEMPLATE = "{prefix}:queue_snapshot:lock"
    LOCK_TIMEOUT = 60       
    SNAPSHOT_INTERVAL = 60  
    CLEANUP_INTERVAL = 3600 
    RETENTION_DAYS = 7      
    HEARTBEAT_TIMEOUT = 15.0  

    def __init__(
        self,
        app: "Jettask",
        async_redis_client,
        pg_session_factory,
        namespace: str
    ):
        self.app = app
        self.redis = async_redis_client
        self.pg_session_factory = pg_session_factory
        self.namespace = namespace
        self._running = False
        self._task: Optional[asyncio.Task] = None
        self._last_cleanup_time = 0

        self.workers_registry_key = f"{app.redis_prefix}:REGISTRY:WORKERS"

    async def start(self):
        self._running = True
        logger.info(f"[QueueSnapshot] 启动队列快照采集服务, namespace={self.namespace}")

        while self._running:
            try:
                await self._collect_with_lock()

                current_time = asyncio.get_event_loop().time()
                if current_time - self._last_cleanup_time >= self.CLEANUP_INTERVAL:
                    await self._cleanup_old_snapshots()
                    self._last_cleanup_time = current_time

            except asyncio.CancelledError:
                logger.info("[QueueSnapshot] 采集任务被取消")
                break
            except Exception as e:
                logger.error(f"[QueueSnapshot] 采集失败: {e}", exc_info=True)

            await asyncio.sleep(self.SNAPSHOT_INTERVAL)

        logger.info("[QueueSnapshot] 队列快照采集服务已停止")

    async def stop(self):
        self._running = False
        if self._task and not self._task.done():
            self._task.cancel()
            try:
                await self._task
            except asyncio.CancelledError:
                pass
        logger.info("[QueueSnapshot] 停止队列快照采集服务")

    async def _collect_with_lock(self):
        lock_key = self.LOCK_KEY_TEMPLATE.format(prefix=self.app.redis_prefix)

        lock = self.redis.lock(
            name=lock_key,
            timeout=self.LOCK_TIMEOUT
        )

        acquired = await lock.acquire(blocking=False)
        if not acquired:
            logger.debug("[QueueSnapshot] 未获取到锁，跳过本次采集")
            return

        try:
            await self._do_collect()
        finally:
            try:
                await lock.release()
            except Exception:
                pass  

    async def _do_collect(self):
        time_bucket = self._get_time_bucket()

        backlog_data = await self._collect_backlog_data(time_bucket)

        worker_data = await self._collect_worker_data()

        snapshots = self._merge_snapshots(time_bucket, backlog_data, worker_data)

        if snapshots:
            await self._batch_upsert(snapshots)
            logger.info(
                f"[QueueSnapshot] 写入 {len(snapshots)} 条快照, "
                f"backlog_data={len(backlog_data)} 条, "
                f"worker_data={len(worker_data)} 组"
            )


    async def _collect_backlog_data(self, time_bucket: datetime) -> List[dict]:
        from jettask.messaging.registry import QueueRegistry

        registry = QueueRegistry(
            redis_client=None,
            async_redis_client=self.redis,
            redis_prefix=self.app.redis_prefix
        )
        task_queue_pairs = await registry.get_all_task_queue_pairs()

        if not task_queue_pairs:
            logger.debug("[QueueSnapshot] 没有任务需要采集积压数据")
            return []

        queues = set(pair['queue'] for pair in task_queue_pairs)
        logger.debug(f"[QueueSnapshot] 采集 {len(queues)} 个队列的积压数据")

        snapshots = []
        for queue in queues:
            try:
                backlog_info = await self.app.get_queue_backlog(queue, asyncio=True)
                logger.debug(f"[QueueSnapshot] 队列 {queue} backlog_info: {backlog_info}")

                if 'error' in backlog_info:
                    logger.warning(f"[QueueSnapshot] 获取队列 {queue} 积压失败: {backlog_info.get('error')}")
                    continue

                send_offset = backlog_info.get('send_offset', 0)

                for task_info in backlog_info.get('tasks', []):
                    snapshots.append({
                        'queue': queue,
                        'task_name': task_info['task_name'],
                        'backlog': task_info.get('backlog', 0),
                        'pending_read': task_info.get('pending_read', 0),
                        'send_offset': send_offset,
                        'ack_offset': task_info.get('ack_offset', 0),
                        'read_offset': task_info.get('read_offset', 0),
                    })

            except Exception as e:
                logger.error(f"[QueueSnapshot] 处理队列 {queue} 积压数据失败: {e}")

        return snapshots


    async def _collect_worker_data(self) -> Dict[tuple, Dict[str, Set[str]]]:
        current_time = time.time()

        try:
            all_worker_ids = await self.redis.smembers(self.workers_registry_key)
            if not all_worker_ids:
                return {}

            pipeline = self.redis.pipeline()
            worker_id_list = list(all_worker_ids)
            for worker_id in worker_id_list:
                wid = worker_id.decode('utf-8') if isinstance(worker_id, bytes) else worker_id
                worker_key = f"{self.app.redis_prefix}:WORKER:{wid}"
                pipeline.hgetall(worker_key)
            worker_data_list = await pipeline.execute()

            result: Dict[tuple, Dict[str, Set[str]]] = defaultdict(lambda: {'online': set(), 'offline': set()})

            for worker_id, worker_data in zip(worker_id_list, worker_data_list):
                if not worker_data:
                    continue

                wid = worker_id.decode('utf-8') if isinstance(worker_id, bytes) else worker_id

                last_heartbeat_raw = worker_data.get('last_heartbeat') or worker_data.get(b'last_heartbeat')
                is_alive_raw = worker_data.get('is_alive') or worker_data.get(b'is_alive')

                last_heartbeat = 0
                if last_heartbeat_raw:
                    val = last_heartbeat_raw.decode('utf-8') if isinstance(last_heartbeat_raw, bytes) else last_heartbeat_raw
                    try:
                        last_heartbeat = float(val)
                    except (ValueError, TypeError):
                        pass

                is_alive = True
                if is_alive_raw:
                    val = is_alive_raw.decode('utf-8') if isinstance(is_alive_raw, bytes) else is_alive_raw
                    is_alive = str(val).lower() == 'true'

                is_online = is_alive and (current_time - last_heartbeat) < self.HEARTBEAT_TIMEOUT
                status = 'online' if is_online else 'offline'

                for key, value in worker_data.items():
                    key_str = key.decode('utf-8') if isinstance(key, bytes) else key
                    if not key_str.startswith('group_info:'):
                        continue

                    try:
                        val_str = value.decode('utf-8') if isinstance(value, bytes) else value
                        group_info = json.loads(val_str)
                        queue = group_info.get('queue')
                        task_name = group_info.get('task_name')
                        if queue and task_name:
                            result[(queue, task_name)][status].add(wid)
                    except (json.JSONDecodeError, Exception) as e:
                        logger.debug(f"[QueueSnapshot] 解析 group_info 失败: {key_str}, error: {e}")
                        continue

            logger.debug(f"[QueueSnapshot] 采集到 {len(result)} 个 (queue, task) Worker 组合")
            return result

        except Exception as e:
            logger.error(f"[QueueSnapshot] 采集 Worker 数据失败: {e}", exc_info=True)
            return {}


    def _merge_snapshots(
        self,
        time_bucket: datetime,
        backlog_data: List[dict],
        worker_data: Dict[tuple, Dict[str, Set[str]]]
    ) -> List[dict]:
        backlog_index: Dict[tuple, dict] = {}
        for item in backlog_data:
            key = (item['queue'], item['task_name'])
            backlog_index[key] = item

        all_keys = set(backlog_index.keys())
        for (queue, task_name) in worker_data:
            all_keys.add((queue, task_name))

        snapshots = []
        empty_backlog = {
            'backlog': 0, 'pending_read': 0,
            'send_offset': 0, 'ack_offset': 0, 'read_offset': 0
        }

        for (queue, task_name) in all_keys:
            backlog_info = backlog_index.get((queue, task_name), empty_backlog)
            workers = worker_data.get((queue, task_name), {'online': set(), 'offline': set()})
            online_count = len(workers['online'])
            offline_count = len(workers['offline'])

            snapshots.append({
                'time_bucket': time_bucket,
                'namespace': self.namespace,
                'queue': queue,
                'task_name': task_name,
                'backlog': backlog_info.get('backlog', 0),
                'pending_read': backlog_info.get('pending_read', 0),
                'send_offset': backlog_info.get('send_offset', 0),
                'ack_offset': backlog_info.get('ack_offset', 0),
                'read_offset': backlog_info.get('read_offset', 0),
                'online_count': online_count,
                'offline_count': offline_count,
                'total_count': online_count + offline_count,
            })

        queue_online: Dict[str, Set[str]] = defaultdict(set)
        queue_offline: Dict[str, Set[str]] = defaultdict(set)
        for (queue, task_name), workers in worker_data.items():
            queue_online[queue].update(workers['online'])
            queue_offline[queue].update(workers['offline'])

        queue_backlog: Dict[str, int] = defaultdict(int)
        queue_pending: Dict[str, int] = defaultdict(int)
        queue_send_offset: Dict[str, int] = defaultdict(int)
        queue_ack_offset: Dict[str, int] = defaultdict(int)
        queue_read_offset: Dict[str, int] = defaultdict(int)
        for item in backlog_data:
            q = item['queue']
            queue_backlog[q] += item.get('backlog', 0)
            queue_pending[q] += item.get('pending_read', 0)
            queue_send_offset[q] += item.get('send_offset', 0)
            queue_ack_offset[q] += item.get('ack_offset', 0)
            queue_read_offset[q] += item.get('read_offset', 0)

        all_queues = set(list(queue_online.keys()) + list(queue_offline.keys()) + list(queue_backlog.keys()))
        for queue in all_queues:
            online = len(queue_online.get(queue, set()))
            offline = len(queue_offline.get(queue, set()))
            snapshots.append({
                'time_bucket': time_bucket,
                'namespace': self.namespace,
                'queue': queue,
                'task_name': '',
                'backlog': queue_backlog.get(queue, 0),
                'pending_read': queue_pending.get(queue, 0),
                'send_offset': queue_send_offset.get(queue, 0),
                'ack_offset': queue_ack_offset.get(queue, 0),
                'read_offset': queue_read_offset.get(queue, 0),
                'online_count': online,
                'offline_count': offline,
                'total_count': online + offline,
            })

        total_backlog = sum(item.get('backlog', 0) for item in backlog_data)
        total_pending_read = sum(item.get('pending_read', 0) for item in backlog_data)
        total_send_offset = sum(item.get('send_offset', 0) for item in backlog_data)
        total_ack_offset = sum(item.get('ack_offset', 0) for item in backlog_data)
        total_read_offset = sum(item.get('read_offset', 0) for item in backlog_data)

        all_online: Set[str] = set()
        all_offline: Set[str] = set()
        for workers in worker_data.values():
            all_online.update(workers['online'])
            all_offline.update(workers['offline'])

        ns_online = len(all_online)
        ns_offline = len(all_offline)
        snapshots.append({
            'time_bucket': time_bucket,
            'namespace': self.namespace,
            'queue': '',
            'task_name': '',
            'backlog': total_backlog,
            'pending_read': total_pending_read,
            'send_offset': total_send_offset,
            'ack_offset': total_ack_offset,
            'read_offset': total_read_offset,
            'online_count': ns_online,
            'offline_count': ns_offline,
            'total_count': ns_online + ns_offline,
        })

        return snapshots


    def _get_time_bucket(self) -> datetime:
        now = datetime.now(timezone.utc)
        return now.replace(second=0, microsecond=0)

    async def _batch_upsert(self, snapshots: List[dict]):
        async with self.pg_session_factory() as session:
            stmt = insert(QueueSnapshot).values(snapshots)
            stmt = stmt.on_conflict_do_update(
                index_elements=['time_bucket', 'namespace', 'queue', 'task_name'],
                set_={
                    'backlog': stmt.excluded.backlog,
                    'pending_read': stmt.excluded.pending_read,
                    'send_offset': stmt.excluded.send_offset,
                    'ack_offset': stmt.excluded.ack_offset,
                    'read_offset': stmt.excluded.read_offset,
                    'online_count': stmt.excluded.online_count,
                    'offline_count': stmt.excluded.offline_count,
                    'total_count': stmt.excluded.total_count,
                }
            )
            await session.execute(stmt)
            await session.commit()

    async def _cleanup_old_snapshots(self):
        cutoff = datetime.now(timezone.utc) - timedelta(days=self.RETENTION_DAYS)

        try:
            async with self.pg_session_factory() as session:
                result = await session.execute(
                    delete(QueueSnapshot).where(
                        QueueSnapshot.time_bucket < cutoff
                    )
                )
                await session.commit()
                deleted_count = result.rowcount
                if deleted_count > 0:
                    logger.info(f"[QueueSnapshot] 清理 {deleted_count} 条过期快照（{self.RETENTION_DAYS} 天前）")
        except Exception as e:
            logger.error(f"[QueueSnapshot] 清理过期快照失败: {e}", exc_info=True)



"""
限流器管理器

统一管理所有任务的限流器实例

职责（简化后）：
1. 管理 TaskRateLimiter 实例
2. 每次 acquire 时从 DynamicConfig 获取最新配置
3. 配置变化时自动重建限流器

不再负责：
- 配置存储和读取（由 DynamicConfig 统一管理）
- 配置变更监听（DynamicConfig 已处理 Pub/Sub）
"""

import asyncio
import logging
from redis.asyncio import Redis
from typing import Dict, Optional, Any, TYPE_CHECKING

from .task_limiter import TaskRateLimiter
from .config import RateLimitConfig, QPSLimit, ConcurrencyLimit

if TYPE_CHECKING:
    from jettask.config.dynamic_config import DynamicConfig

logger = logging.getLogger('app')


class RateLimiterManager:
    """限流器管理器

    管理多个任务的限流器。
    每次 acquire 时从 DynamicConfig 获取最新配置，自动检测配置变化并重建限流器。
    """

    def __init__(
        self,
        redis_client: Redis,
        worker_id: str,
        redis_prefix: str = "jettask",
        dynamic_config: 'DynamicConfig' = None,
        worker_state_manager=None
    ):
        self._redis = redis_client
        self._worker_id = worker_id
        self._prefix = redis_prefix
        self._dynamic_config = dynamic_config
        self._worker_state_manager = worker_state_manager

        self._limiters: Dict[str, TaskRateLimiter] = {}

        self._config_snapshots: Dict[str, Optional[Dict]] = {}

        self._rebuild_locks: Dict[str, asyncio.Lock] = {}

        logger.debug(f"RateLimiterManager initialized for worker {worker_id}")

    def set_dynamic_config(self, dynamic_config: 'DynamicConfig'):
        self._dynamic_config = dynamic_config
        logger.debug("DynamicConfig injected into RateLimiterManager")

    async def preload_all(self):
        if not self._dynamic_config:
            logger.warning("[RateLimiter] DynamicConfig 未设置，跳过预加载")
            return

        task_names = self._dynamic_config.get_all_task_names()
        if not task_names:
            logger.debug("[RateLimiter] 没有已配置的任务，跳过预加载")
            return

        loaded_count = 0
        for task_name in task_names:
            config = self._dynamic_config.get_rate_limit(task_name)
            if config:
                try:
                    await self._rebuild_limiter(task_name, config)
                    loaded_count += 1
                except Exception as e:
                    logger.error(f"[RateLimiter] 预加载限流器失败: {task_name}, error: {e}")

        logger.info(f"[RateLimiter] 预加载完成，已加载 {loaded_count} 个限流器")

    def _get_rebuild_lock(self, task_name: str) -> asyncio.Lock:
        if task_name not in self._rebuild_locks:
            self._rebuild_locks[task_name] = asyncio.Lock()
        return self._rebuild_locks[task_name]

    async def acquire(self, task_name: str, timeout: float = None) -> Optional[str]:
        config = self._dynamic_config.get_rate_limit(task_name) if self._dynamic_config else None

        if self._need_rebuild(task_name, config):
            lock = self._get_rebuild_lock(task_name)
            async with lock:
                if self._need_rebuild(task_name, config):
                    await self._rebuild_limiter(task_name, config)

        limiter = self._limiters.get(task_name)
        if limiter:
            return await limiter.acquire(timeout=timeout)

        return True

    async def release(self, task_name: str, task_id: Optional[str] = None):
        limiter = self._limiters.get(task_name)
        if limiter:
            await limiter.release(task_id)

    def _need_rebuild(self, task_name: str, new_config: Optional[Dict]) -> bool:
        old_config = self._config_snapshots.get(task_name)

        if old_config is None and new_config is not None:
            return True

        if old_config is not None and new_config is None:
            return True

        if old_config != new_config:
            return True

        if new_config is not None and task_name not in self._limiters:
            return True

        return False

    async def _rebuild_limiter(self, task_name: str, config: Optional[Dict]):
        old_limiter = self._limiters.pop(task_name, None)
        if old_limiter:
            try:
                await old_limiter.stop()
            except Exception as e:
                logger.warning(f"[RateLimiter] 停止旧限流器失败: {task_name}, error: {e}")

        self._config_snapshots[task_name] = config

        if config is None:
            logger.info(f"[RateLimiter] 移除限流器: {task_name}")
            return

        rate_limit_config = self._parse_config(config)
        if rate_limit_config:
            limiter = TaskRateLimiter(
                redis_client=self._redis,
                task_name=task_name,
                worker_id=self._worker_id,
                config=rate_limit_config,
                redis_prefix=self._prefix,
                worker_state_manager=self._worker_state_manager
            )
            await limiter.start()
            self._limiters[task_name] = limiter
            logger.info(f"[RateLimiter] 重建限流器: {task_name} = {config}")

    def _parse_config(self, config: Dict) -> Optional[RateLimitConfig]:
        rate_type = config.get("type")
        if rate_type == "qps":
            return QPSLimit(
                qps=config.get("qps", 10),
                window_size=config.get("window_size", 1.0)
            )
        elif rate_type == "concurrency":
            return ConcurrencyLimit(
                max_concurrency=config.get("max_concurrency", 10)
            )
        return None

    async def stop_all(self):
        for task_name, limiter in list(self._limiters.items()):
            try:
                await limiter.stop()
            except Exception as e:
                logger.warning(f"[RateLimiter] 停止限流器失败: {task_name}, error: {e}")
        self._limiters.clear()
        self._config_snapshots.clear()
        logger.debug("Stopped all rate limiters")

    def get_all_stats(self) -> Dict[str, dict]:
        stats = {}
        for task_name, limiter in self._limiters.items():
            if hasattr(limiter, 'get_stats'):
                stats[task_name] = limiter.get_stats()
        return stats


    @staticmethod
    def register_rate_limit_config(
        redis_client,
        task_name: str,
        config: RateLimitConfig,
        redis_prefix: str = "jettask",
        force: bool = False
    ):
        try:
            config_key = f"{redis_prefix}:CONFIG:TASK:{task_name}"
            index_key = f"{redis_prefix}:CONFIG:TASK:INDEX"
            version_key = f"{redis_prefix}:CONFIG:VERSION"

            if not force:
                existing_type = redis_client.hget(config_key, "rate_limit_type")
                if existing_type:
                    logger.debug(
                        f"任务 '{task_name}' 已有限流配置（可能来自 WebUI），跳过代码中的默认配置"
                    )
                    return

            config_dict = config.to_dict()
            flat_config = {}
            rate_type = config_dict.get("type")
            flat_config["rate_limit_type"] = rate_type

            if rate_type == "qps":
                flat_config["rate_limit_qps"] = str(config_dict.get("qps", 10))
                flat_config["rate_limit_window_size"] = str(config_dict.get("window_size", 1.0))
            elif rate_type == "concurrency":
                flat_config["rate_limit_max_concurrency"] = str(config_dict.get("max_concurrency", 10))

            redis_client.hset(config_key, mapping=flat_config)
            redis_client.sadd(index_key, task_name)
            redis_client.incr(version_key)

            logger.debug(f"Registered rate limit config for task '{task_name}': {config}")
        except Exception as e:
            logger.error(f"Failed to register rate limit config for task '{task_name}': {e}")

    @staticmethod
    def unregister_rate_limit_config(
        redis_client,
        task_name: str,
        redis_prefix: str = "jettask"
    ):
        try:
            config_key = f"{redis_prefix}:CONFIG:TASK:{task_name}"
            index_key = f"{redis_prefix}:CONFIG:TASK:INDEX"
            version_key = f"{redis_prefix}:CONFIG:VERSION"

            redis_client.hdel(config_key, "rate_limit_type", "rate_limit_qps",
                             "rate_limit_window_size", "rate_limit_max_concurrency")

            remaining = redis_client.hlen(config_key)
            if remaining == 0:
                redis_client.delete(config_key)
                redis_client.srem(index_key, task_name)

            redis_client.incr(version_key)

            logger.debug(f"Removed rate limit config for task '{task_name}'")
            return True
        except Exception as e:
            logger.error(f"Failed to remove rate limit config for task '{task_name}': {e}")
            return False


__all__ = ['RateLimiterManager']



"""
Jettask v2 通用类型定义
"""

from typing import Any, Dict, List, Optional
from enum import Enum


class TaskStatus(str, Enum):
    """任务状态"""

    PENDING = "PENDING"
    STARTED = "STARTED"
    SUCCESS = "SUCCESS"
    FAILURE = "FAILURE"
    REVOKED = "REVOKED"
    IGNORED = "IGNORED"


class WorkerStatus(str, Enum):
    """Worker 状态"""

    ONLINE = "ONLINE"
    OFFLINE = "OFFLINE"
    TIMEOUT = "TIMEOUT"


class ScheduleStatus(str, Enum):
    """调度状态"""

    ACTIVE = "ACTIVE"
    PAUSED = "PAUSED"
    COMPLETED = "COMPLETED"
    FAILED = "FAILED"


class TriggerType(str, Enum):
    """触发器类型"""

    CRON = "cron"
    INTERVAL = "interval"
    DATE = "date"


class QueueType(str, Enum):
    """队列类型"""

    REDIS = "redis"
    RABBITMQ = "rabbitmq"
    KAFKA = "kafka"



"""
WebUI 通用依赖 - 提供带类型注解的依赖函数

使用方法:
```python
from jettask.webui.deps import get_ns

@router.get("/example")
async def example(ns: NamespaceContext = Depends(get_ns)):
    # ns 现在有完整的类型提示
    redis = await ns.get_redis_client()
    ...
```

或者使用 Annotated 语法（推荐）:
```python
from jettask.webui.deps import NsDep

@router.get("/example")
async def example(ns: NsDep):
    # ns 现在有完整的类型提示
    redis = await ns.get_redis_client()
    ...
```
"""
from typing import Annotated
from fastapi import Request, Depends, HTTPException

from jettask.core.namespace import NamespaceContext


def get_ns(request: Request) -> NamespaceContext:
    if not hasattr(request.state, 'ns'):
        raise HTTPException(
            status_code=500,
            detail="Namespace context not initialized. This endpoint requires a namespace."
        )
    return request.state.ns


NsDep = Annotated[NamespaceContext, Depends(get_ns)]



"""
设置模块 - 系统配置
提供轻量级的路由入口，业务逻辑在 SettingsService 中实现
"""
from fastapi import APIRouter, HTTPException, Request, Body
from pydantic import BaseModel, Field
from typing import Dict, Any, Optional
import logging
import traceback

from jettask.schemas import SystemSettingsResponse, DatabaseStatusResponse
from jettask.webui.services.settings_service import SettingsService
from jettask.config import SYSTEM_CONFIG_DEFAULTS

logger = logging.getLogger(__name__)

router = APIRouter(prefix="/settings", tags=["设置"])



class DynamicConfigResponse(BaseModel):
    """动态配置响应"""
    success: bool = True
    data: Dict[str, Any]


class UpdateConfigRequest(BaseModel):
    """更新配置请求"""
    value: Any = Field(..., description="配置值")
    description: Optional[str] = Field(None, description="配置描述")



@router.get(
    "/system",
    summary="获取系统配置信息",
    description="获取系统级别的配置信息，包括 API 版本、服务名称、运行环境、数据库状态等",
    response_model=SystemSettingsResponse,
    responses={
        200: {
            "description": "成功返回系统配置信息"
        },
        500: {
            "description": "服务器内部错误",
            "content": {
                "application/json": {
                    "example": {
                        "detail": "获取系统配置失败: Configuration error"
                    }
                }
            }
        }
    }
)
async def get_system_settings() -> Dict[str, Any]:
    try:
        return SettingsService.get_system_settings()
    except Exception as e:
        logger.error(f"获取系统配置失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))


@router.get(
    "/database-status",
    summary="检查数据库连接状态",
    description="检查数据库（PostgreSQL/MySQL）的连接状态和性能指标",
    response_model=DatabaseStatusResponse,
    responses={
        200: {
            "description": "成功返回数据库状态"
        },
        500: {
            "description": "服务器内部错误或数据库连接失败",
            "content": {
                "application/json": {
                    "example": {
                        "detail": "数据库状态检查失败: Connection refused"
                    }
                }
            }
        }
    }
)
async def check_database_status() -> Dict[str, Any]:
    try:
        return await SettingsService.check_database_status()
    except Exception as e:
        logger.error(f"数据库状态检查失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))




async def _get_namespace_manager(request: Request):
    if not hasattr(request.app.state, 'namespace_manager'):
        raise HTTPException(
            status_code=500,
            detail="NamespaceManager 未初始化"
        )
    return request.app.state.namespace_manager


async def _get_system_configs_from_redis(request: Request) -> Dict[str, Any]:
    from jettask.config import DynamicConfig
    import json

    namespace_manager = await _get_namespace_manager(request)
    namespaces = await namespace_manager.list_namespaces(enabled_only=True)

    if not namespaces:
        return {}

    ns = namespaces[0]
    redis_client = await ns.get_redis_client()
    prefix = ns.redis_prefix

    config_key = DynamicConfig.get_system_config_key(prefix)
    data = await redis_client.hgetall(config_key)

    configs = {}
    for k, v in data.items():
        k = k.decode() if isinstance(k, bytes) else k
        v = v.decode() if isinstance(v, bytes) else v
        try:
            configs[k] = json.loads(v)
        except (json.JSONDecodeError, TypeError):
            configs[k] = v

    return configs


@router.get(
    "/configs",
    summary="获取所有动态配置",
    description="获取所有系统级动态配置，包括 DDSketch 精度、指标保留天数等",
    response_model=DynamicConfigResponse,
)
async def get_all_configs(request: Request) -> Dict[str, Any]:
    try:
        configs = dict(SYSTEM_CONFIG_DEFAULTS)
        redis_configs = await _get_system_configs_from_redis(request)
        configs.update(redis_configs)

        return {"success": True, "data": configs}
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"获取动态配置失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))


@router.get(
    "/configs/{key}",
    summary="获取单个动态配置",
    description="获取指定键的动态配置值",
    response_model=DynamicConfigResponse,
)
async def get_config(request: Request, key: str) -> Dict[str, Any]:
    try:
        redis_configs = await _get_system_configs_from_redis(request)
        value = redis_configs.get(key) or SYSTEM_CONFIG_DEFAULTS.get(key)

        if value is None:
            raise HTTPException(status_code=404, detail=f"配置 '{key}' 不存在")

        return {"success": True, "data": {key: value}}
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"获取配置 {key} 失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))


@router.put(
    "/configs/{key}",
    summary="更新动态配置",
    description="更新指定键的动态配置值",
    response_model=DynamicConfigResponse,
)
async def update_config(
    request: Request,
    key: str,
    body: UpdateConfigRequest
) -> Dict[str, Any]:
    from jettask.config import DynamicConfig
    from jettask.webui.services.config_service import ConfigService

    try:
        if key not in SYSTEM_CONFIG_DEFAULTS:
            raise HTTPException(
                status_code=400,
                detail=f"无效的配置键 '{key}'，有效的配置键: {list(SYSTEM_CONFIG_DEFAULTS.keys())}"
            )

        namespace_manager = await _get_namespace_manager(request)
        namespaces = await namespace_manager.list_namespaces(enabled_only=True)

        for ns in namespaces:
            try:
                redis_client = await ns.get_redis_client()
                prefix = ns.redis_prefix
                await DynamicConfig.set_system_config_to_redis(
                    redis_client, prefix, key, body.value
                )
                logger.debug(f"系统配置 {key} 已同步到命名空间 {ns.name}")
            except Exception as e:
                logger.error(f"同步系统配置到命名空间 {ns.name} 失败: {e}")

        meta_db_session_factory = request.app.state.meta_db_session_factory
        async with meta_db_session_factory() as pg_session:
            await ConfigService.save_system_config_to_pg(
                pg_session, key, body.value, body.description
            )

        return {
            "success": True,
            "data": {key: body.value},
        }
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"更新配置 {key} 失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))


@router.get(
    "/configs-schema",
    summary="获取配置项定义",
    description="获取所有可配置项的定义，包括类型、默认值、说明",
)
async def get_configs_schema() -> Dict[str, Any]:
    schema = {
        "ddsketch_relative_accuracy": {
            "type": "float",
            "default": 0.05,
            "min": 0.01,
            "max": 0.1,
            "description": "DDSketch 相对误差（用于分位数计算），值越小精度越高但存储越大"
        },
        "metrics_retention_days": {
            "type": "int",
            "default": 30,
            "min": 1,
            "max": 365,
            "description": "指标数据保留天数"
        },
        "log_level": {
            "type": "string",
            "default": "INFO",
            "options": ["DEBUG", "INFO", "WARNING", "ERROR"],
            "description": "日志级别"
        }
    }
    return {"success": True, "data": schema}


__all__ = ['router']


"""
队列模块 - 队列管理、任务处理、队列统计和监控、任务发送
提供轻量级的路由入口，业务逻辑在 QueueService 中实现
包含跨语言任务发送 API
"""
from fastapi import APIRouter, HTTPException, Request, Query, Path, Depends
from typing import Optional, List, Dict, Any
from datetime import datetime

from jettask.schemas import (
    SendTasksRequest,
    SendTasksResponse
)
from jettask.core.message import TaskMessage
from jettask.core.namespace import NamespaceContext
from jettask.webui.deps import NsDep, get_ns
from jettask.webui.services.metrics_service import MetricsService, RunsMetric
from jettask.webui.services.queue_service import QueueService
import base64
import json
from jettask.utils.task_logger import LogContext, get_task_logger

router = APIRouter(prefix="/queues", tags=["队列"])
logger = get_task_logger(__name__)



@router.get(
    "",
    summary="获取命名空间队列列表",
    description="""获取指定命名空间下所有基础队列的列表。

**主要功能**:
- 从 QueueRegistry 获取所有注册的队列
- 自动过滤优先级队列，只返回基础队列名称
- 实时从 Redis 获取数据

**使用场景**: 队列管理页面列表、队列状态监控、队列选择器、管理界面展示""",
    responses={
        200: {
            "description": "成功返回队列列表",
            "content": {
                "application/json": {
                    "example": {
                        "success": True,
                        "namespace": "default",
                        "queues": [
                            {"name": "email_queue"},
                            {"name": "sms_queue"}
                        ],
                        "total": 2
                    }
                }
            }
        },
        500: {"description": "服务器内部错误"}
    }
)
async def get_queues(ns: NamespaceContext = Depends(get_ns)) -> Dict[str, Any]:
    try:
        registry = await ns.get_queue_registry()

        base_queues = await registry.get_base_queues()

        queues_list = []
        for queue_name in sorted(base_queues):
            if queue_name == "TASK_CHANGES":
                continue
            queues_list.append({
                "name": queue_name
            })

        logger.info(f"获取命名空间 {ns.name} 的队列列表成功，共 {len(queues_list)} 个队列")

        return {
            "success": True,
            "namespace": ns.name,
            "queues": queues_list,
            "total": len(queues_list)
        }

    except Exception as e:
        logger.error(f"获取队列列表失败: {e}", exc_info=True)
        raise HTTPException(status_code=500, detail=str(e))




@router.get(
    "/{queue}/backlog",
    summary="获取队列积压情况",
    description="""获取指定队列的任务积压数量，用于监控队列健康状态。

**积压计算**:
- 积压数 = 发送offset - 消费确认offset
- pending_read = 发送offset - 读取offset

**返回字段说明**:
- total_backlog: 总积压数量（待消费确认的任务数）
- total_pending_read: 待读取数量（已发送但未被 Worker 读取的任务数）
- send_offset: 队列的发送 offset（累计发送消息数）
- tasks: 每个任务类型的详细积压情况

**使用场景**: 队列健康监控、积压告警、容量规划""",
    responses={
        200: {
            "description": "成功返回队列积压数据",
            "content": {
                "application/json": {
                    "examples": {
                        "队列总积压": {
                            "value": {
                                "success": True,
                                "queue": "email_queue",
                                "total_backlog": 156,
                                "total_pending_read": 20,
                                "send_offset": 10000,
                                "tasks": [
                                    {
                                        "task_name": "send_email",
                                        "ack_offset": 9800,
                                        "backlog": 100,
                                        "read_offset": 9900,
                                        "pending_read": 10
                                    },
                                    {
                                        "task_name": "send_sms",
                                        "ack_offset": 9944,
                                        "backlog": 56,
                                        "read_offset": 9990,
                                        "pending_read": 10
                                    }
                                ]
                            }
                        },
                        "指定任务积压": {
                            "value": {
                                "success": True,
                                "queue": "email_queue",
                                "task_name": "send_email",
                                "send_offset": 10000,
                                "ack_offset": 9800,
                                "backlog": 200,
                                "read_offset": 9900,
                                "pending_read": 100
                            }
                        }
                    }
                }
            }
        },
        500: {"description": "服务器内部错误"}
    }
)
async def get_queue_backlog(
    ns: NsDep,
    queue: str = Path(..., description="队列名称", example="email_queue"),
    task_name: Optional[str] = Query(None, description="任务名称（可选，不指定则返回所有任务的积压）")
) -> Dict[str, Any]:
    try:
        app = await ns.get_app()
        result = await app.get_queue_backlog(queue, task_name=task_name, asyncio=True)

        if "error" in result:
            raise HTTPException(status_code=500, detail=result["error"])

        return {
            "success": True,
            **result
        }

    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"获取队列积压失败: {e}", exc_info=True)
        raise HTTPException(status_code=500, detail=str(e))




@router.get(
    "/backlog/trend",
    summary="获取积压趋势数据",
    description="""从 queue_snapshot 表查询积压趋势数据。

**数据来源**:
- queue_snapshot 表，每分钟采集一次
- 与任务执行解耦，即使没有任务执行也能记录积压

**聚合模式**:
- `aggregate_by_queue=true`: 返回队列级汇总
- `aggregate_by_queue=false`: 返回任务级明细

**时间间隔自动计算**:
- ≤ 1 小时：1 分钟
- 1-6 小时：5 分钟
- 6-24 小时：15 分钟
- 1-7 天：1 小时
- > 7 天：6 小时

**使用场景**: 积压监控、容量规划、告警分析""",
    responses={
        200: {
            "description": "成功返回积压趋势数据",
            "content": {
                "application/json": {
                    "examples": {
                        "队列级汇总": {
                            "value": {
                                "success": True,
                                "data": [
                                    {
                                        "queue": "email_queue",
                                        "trend": [
                                            {"time": "2026-01-22T10:00:00Z", "backlog": 100, "pending_read": 20},
                                            {"time": "2026-01-22T10:01:00Z", "backlog": 95, "pending_read": 15}
                                        ]
                                    }
                                ],
                                "granularity": "minute",
                                "time_range": {
                                    "start_time": "2026-01-22T10:00:00Z",
                                    "end_time": "2026-01-22T11:00:00Z",
                                    "interval": "1 minute",
                                    "interval_seconds": 60
                                }
                            }
                        },
                        "任务级明细": {
                            "value": {
                                "success": True,
                                "data": [
                                    {
                                        "queue": "email_queue",
                                        "tasks": [
                                            {
                                                "task_name": "send_email",
                                                "trend": [
                                                    {"time": "2026-01-22T10:00:00Z", "backlog": 50, "pending_read": 10}
                                                ]
                                            }
                                        ]
                                    }
                                ]
                            }
                        }
                    }
                }
            }
        },
        400: {"description": "请求参数错误"},
        500: {"description": "服务器内部错误"}
    }
)
async def get_backlog_trend(
    ns: NsDep,
    queues: str = Query(..., description="队列名称，多个用逗号分隔", example="email_queue,sms_queue"),
    task_names: Optional[str] = Query(None, description="任务名称筛选，多个用逗号分隔"),
    start_time: Optional[datetime] = Query(None, description="开始时间（ISO格式）"),
    end_time: Optional[datetime] = Query(None, description="结束时间（ISO格式）"),
    time_range: Optional[str] = Query(None, description="时间范围，如 1h, 24h, 7d", example="1h"),
    aggregate_by_queue: bool = Query(False, description="是否按队列聚合（True 返回队列级汇总，False 返回任务级明细）")
) -> Dict[str, Any]:
    try:
        pg_session = await ns.get_pg_session()

        try:
            queue_list = [q.strip() for q in queues.split(',') if q.strip()]
            if not queue_list:
                raise HTTPException(status_code=400, detail="queues 参数不能为空")

            task_name_list = None
            if task_names:
                task_name_list = [t.strip() for t in task_names.split(',') if t.strip()]

            result = await QueueService.get_backlog_trend(
                namespace=ns.name,
                pg_session=pg_session,
                queues=queue_list,
                task_names=task_name_list,
                start_time=start_time,
                end_time=end_time,
                time_range=time_range,
                aggregate_by_queue=aggregate_by_queue
            )
            return result

        finally:
            await pg_session.close()

    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"获取积压趋势失败: {e}", exc_info=True)
        raise HTTPException(status_code=500, detail=str(e))


@router.get(
    "/backlog/summary",
    summary="获取当前积压汇总",
    description="""获取最新的积压汇总数据。

**数据来源**:
- 查询 queue_snapshot 表中最新一条快照
- 返回各队列各任务的当前积压情况

**使用场景**: 仪表盘展示、实时监控""",
    responses={
        200: {
            "description": "成功返回积压汇总数据",
            "content": {
                "application/json": {
                    "example": {
                        "success": True,
                        "data": [
                            {
                                "queue": "email_queue",
                                "task_name": "send_email",
                                "backlog": 100,
                                "pending_read": 20,
                                "send_offset": 10000,
                                "ack_offset": 9900,
                                "read_offset": 9980
                            }
                        ],
                        "latest_time": "2026-01-22T10:05:00Z"
                    }
                }
            }
        },
        400: {"description": "请求参数错误"},
        500: {"description": "服务器内部错误"}
    }
)
async def get_backlog_summary(
    ns: NsDep,
    queues: str = Query(..., description="队列名称，多个用逗号分隔", example="email_queue,sms_queue"),
    task_names: Optional[str] = Query(None, description="任务名称筛选，多个用逗号分隔")
) -> Dict[str, Any]:
    try:
        pg_session = await ns.get_pg_session()

        try:
            queue_list = [q.strip() for q in queues.split(',') if q.strip()]
            if not queue_list:
                raise HTTPException(status_code=400, detail="queues 参数不能为空")

            task_name_list = None
            if task_names:
                task_name_list = [t.strip() for t in task_names.split(',') if t.strip()]

            result = await QueueService.get_backlog_summary(
                namespace=ns.name,
                pg_session=pg_session,
                queues=queue_list,
                task_names=task_name_list
            )
            return result

        finally:
            await pg_session.close()

    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"获取积压汇总失败: {e}", exc_info=True)
        raise HTTPException(status_code=500, detail=str(e))




@router.get(
    "/{queue}/analytics/duration-distribution",
    summary="获取任务耗时分布",
    description="""获取指定队列的任务耗时分布数据，包括分位数（P50/P90/P99）和直方图。

**数据来源**:
- 从 task_runs_metrics_minute 聚合表查询直方图桶数据
- 分位数基于直方图桶数据估算（线性插值法）

**直方图桶划分**:
- 0 ~ 100ms
- 100ms ~ 500ms
- 500ms ~ 1s
- 1s ~ 5s
- > 5s

**使用场景**: 性能监控、SLA 分析、瓶颈定位""",
    responses={
        200: {
            "description": "成功返回耗时分布数据",
            "content": {
                "application/json": {
                    "example": {
                        "success": True,
                        "percentiles": {
                            "p50": 156.0,
                            "p90": 890.0,
                            "p99": 2300.0
                        },
                        "histogram": [
                            {"range": "0-100ms", "count": 1200},
                            {"range": "100-500ms", "count": 800},
                            {"range": "500ms-1s", "count": 300},
                            {"range": "1s-5s", "count": 100},
                            {"range": ">5s", "count": 20}
                        ],
                        "time_range": {
                            "start_time": "2025-01-22T00:00:00Z",
                            "end_time": "2025-01-22T23:59:59Z"
                        }
                    }
                }
            }
        },
        400: {"description": "请求参数错误"},
        500: {"description": "服务器内部错误"}
    }
)
async def get_duration_distribution(
    ns: NsDep,
    queue: str = Path(..., description="队列名称", example="email_queue"),
    task_names: Optional[str] = Query(None, description="任务名称，多个用逗号分隔"),
    start_time: Optional[datetime] = Query(None, description="开始时间（ISO格式）"),
    end_time: Optional[datetime] = Query(None, description="结束时间（ISO格式）"),
    time_range: Optional[str] = Query(None, description="时间范围：1h/6h/12h/24h/7d/30d", example="1h")
) -> Dict[str, Any]:
    try:
        registry = await ns.get_queue_registry()
        pg_session = await ns.get_pg_session()

        task_name_list = [t.strip() for t in task_names.split(",")] if task_names else None

        result = await MetricsService.get_duration_distribution(
            namespace=ns.name,
            pg_session=pg_session,
            registry=registry,
            queue=queue,
            task_names=task_name_list,
            start_time=start_time,
            end_time=end_time,
            time_range=time_range
        )

        if not result.get("success", True):
            raise HTTPException(status_code=500, detail=result.get("error", "未知错误"))

        return result

    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"获取耗时分布失败: {e}", exc_info=True)
        raise HTTPException(status_code=500, detail=str(e))


@router.get(
    "/{queue}/analytics/error-stats",
    summary="获取错误类型统计",
    description="""获取指定队列的错误类型统计（Top N）。

**数据来源**:
- 从 task_runs 表查询 error_type 字段
- error_type 在任务执行失败时自动记录（异常类名）

**使用场景**: 错误分析、问题定位、稳定性监控""",
    responses={
        200: {
            "description": "成功返回错误类型统计数据",
            "content": {
                "application/json": {
                    "example": {
                        "success": True,
                        "data": [
                            {"error_type": "TimeoutError", "count": 45, "percentage": 45.0},
                            {"error_type": "ValueError", "count": 23, "percentage": 23.0},
                            {"error_type": "ConnectionError", "count": 18, "percentage": 18.0},
                            {"error_type": "RateLimitError", "count": 10, "percentage": 10.0},
                            {"error_type": "其他", "count": 4, "percentage": 4.0}
                        ],
                        "total_errors": 100,
                        "time_range": {
                            "start_time": "2025-01-22T00:00:00Z",
                            "end_time": "2025-01-22T23:59:59Z"
                        }
                    }
                }
            }
        },
        400: {"description": "请求参数错误"},
        500: {"description": "服务器内部错误"}
    }
)
async def get_error_stats(
    ns: NsDep,
    queue: str = Path(..., description="队列名称", example="email_queue"),
    task_names: Optional[str] = Query(None, description="任务名称，多个用逗号分隔"),
    start_time: Optional[datetime] = Query(None, description="开始时间（ISO格式）"),
    end_time: Optional[datetime] = Query(None, description="结束时间（ISO格式）"),
    time_range: Optional[str] = Query(None, description="时间范围：1h/6h/12h/24h/7d/30d", example="1h"),
    limit: int = Query(5, description="返回的错误类型数量", ge=1, le=20)
) -> Dict[str, Any]:
    try:
        registry = await ns.get_queue_registry()
        pg_session = await ns.get_pg_session()

        task_name_list = [t.strip() for t in task_names.split(",")] if task_names else None

        result = await MetricsService.get_error_stats(
            namespace=ns.name,
            pg_session=pg_session,
            registry=registry,
            queue=queue,
            task_names=task_name_list,
            start_time=start_time,
            end_time=end_time,
            time_range=time_range,
            limit=limit
        )

        if not result.get("success", True):
            raise HTTPException(status_code=500, detail=result.get("error", "未知错误"))

        return result

    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"获取错误类型统计失败: {e}", exc_info=True)
        raise HTTPException(status_code=500, detail=str(e))




@router.get(
    "/metrics/tasks/summary",
    summary="任务提交汇总统计",
    description="""从 task_metrics_minute 聚合表查询任务提交/入队的汇总统计。

**功能说明**:
- 统计时间范围内各队列的任务提交总量
- 自动将基础队列名展开为所有优先级队列

**使用场景**: 流量监控、容量规划、队列负载分析""",
    responses={
        200: {
            "description": "成功返回任务提交汇总数据",
            "content": {
                "application/json": {
                    "example": {
                        "success": True,
                        "data": [
                            {"queue_name": "email_queue", "task_count": 15000},
                            {"queue_name": "sms_queue", "task_count": 8000}
                        ],
                        "time_range": {
                            "start_time": "2025-01-22T00:00:00Z",
                            "end_time": "2025-01-22T23:59:59Z"
                        }
                    }
                }
            }
        },
        400: {"description": "请求参数错误"},
        500: {"description": "服务器内部错误"}
    }
)
async def get_metrics_tasks_summary(
    ns: NsDep,
    queues: str = Query(..., description="队列名称，多个用逗号分隔", example="email_queue,sms_queue"),
    start_time: Optional[datetime] = Query(None, description="开始时间（ISO格式）"),
    end_time: Optional[datetime] = Query(None, description="结束时间（ISO格式）"),
    time_range: Optional[str] = Query(None, description="时间范围，如 15m, 1h, 24h, 7d", example="1h")
) -> Dict[str, Any]:
    try:
        pg_session = await ns.get_pg_session()
        registry = await ns.get_queue_registry()

        try:
            queue_list = [q.strip() for q in queues.split(',') if q.strip()]
            if not queue_list:
                raise HTTPException(status_code=400, detail="queues 参数不能为空")

            result = await MetricsService.get_tasks_summary(
                namespace=ns.name,
                pg_session=pg_session,
                registry=registry,
                queues=queue_list,
                start_time=start_time,
                end_time=end_time,
                time_range=time_range
            )
            return result

        finally:
            await pg_session.close()

    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"获取任务提交汇总失败: {e}", exc_info=True)
        raise HTTPException(status_code=500, detail=str(e))


@router.get(
    "/metrics/tasks/trend",
    summary="任务提交时序趋势",
    description="""从 task_metrics_minute 聚合表查询任务提交的时序趋势数据。

**功能说明**:
- 按时间粒度返回任务提交数量的时序数据
- 时间间隔自动根据查询范围计算
- 自动将基础队列名展开为所有优先级队列

**时间间隔自动计算**:
- ≤ 1 小时：1 分钟
- 1-6 小时：5 分钟
- 6-24 小时：15 分钟
- 1-7 天：1 小时
- > 7 天：6 小时

**使用场景**: 任务提交速率监控、流量分析、峰值检测""",
    responses={
        200: {
            "description": "成功返回任务提交趋势数据",
            "content": {
                "application/json": {
                    "example": {
                        "success": True,
                        "data": [
                            {
                                "queue_name": "email_queue",
                                "trend": [
                                    {"time": "2025-01-22T10:00:00Z", "task_count": 100},
                                    {"time": "2025-01-22T10:05:00Z", "task_count": 150}
                                ]
                            }
                        ],
                        "granularity": "5m",
                        "time_range": {
                            "start_time": "2025-01-22T10:00:00Z",
                            "end_time": "2025-01-22T11:00:00Z",
                            "interval": "5 minutes",
                            "interval_seconds": 300
                        }
                    }
                }
            }
        },
        400: {"description": "请求参数错误"},
        500: {"description": "服务器内部错误"}
    }
)
async def get_metrics_tasks_trend(
    ns: NsDep,
    queues: str = Query(..., description="队列名称，多个用逗号分隔", example="email_queue,sms_queue"),
    task_names: Optional[str] = Query(None, description="任务名称筛选，多个用逗号分隔"),
    start_time: Optional[datetime] = Query(None, description="开始时间（ISO格式）"),
    end_time: Optional[datetime] = Query(None, description="结束时间（ISO格式）"),
    time_range: Optional[str] = Query(None, description="时间范围，如 15m, 1h, 24h, 7d", example="1h")
) -> Dict[str, Any]:
    try:
        pg_session = await ns.get_pg_session()
        registry = await ns.get_queue_registry()

        try:
            queue_list = [q.strip() for q in queues.split(',') if q.strip()]
            if not queue_list:
                raise HTTPException(status_code=400, detail="queues 参数不能为空")

            task_name_list = None
            if task_names:
                task_name_list = [t.strip() for t in task_names.split(',') if t.strip()]

            result = await MetricsService.get_tasks_trend(
                namespace=ns.name,
                pg_session=pg_session,
                registry=registry,
                queues=queue_list,
                task_names=task_name_list,
                start_time=start_time,
                end_time=end_time,
                time_range=time_range
            )
            return result

        finally:
            await pg_session.close()

    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"获取任务提交趋势失败: {e}", exc_info=True)
        raise HTTPException(status_code=500, detail=str(e))


@router.get(
    "/metrics/runs/summary",
    summary="任务执行汇总统计",
    description="""从 task_runs_metrics_minute 聚合表查询任务执行情况的汇总统计。

**功能说明**:
- 统计时间范围内各队列下每个任务类型的执行情况
- 支持指定返回的指标字段
- 支持 cursor 分页（任务类型较多时）
- 支持按队列聚合（aggregate_tasks=true 时忽略 task_name 维度）

**支持的指标**:
- 计数类：total_count, success_count, failed_count, retry_count
- 耗时类：avg_duration, max_duration, min_duration（单位：秒）
- 延迟类：avg_delay, max_delay, min_delay（单位：秒，表示任务从创建到开始执行的等待时间）
- 并发类：max_concurrency（单位：任务数，表示单分钟内开始执行的任务数峰值，非实时并发）
- 计算类：success_rate（单位：百分比）

**指标详细说明**:
- total_count: 总执行次数（一个任务可能因重试被执行多次）
- success_count / failed_count: 成功/失败的执行次数
- retry_count: 重试次数累计
- avg_duration: 平均执行耗时（秒），从任务开始执行到完成的时间
- max_duration / min_duration: 执行耗时的最大/最小值（秒）
- avg_delay: 平均延迟（秒），从任务触发时间到实际开始执行的等待时间
- max_delay / min_delay: 延迟的最大/最小值（秒）
- max_concurrency: 单分钟内开始执行的任务数峰值（注意：这是近似并发指标，表示某分钟内有多少任务开始执行，而非同一时刻正在运行的任务数）
- success_rate: 成功率 = success_count / (success_count + failed_count) * 100

**aggregate_tasks 参数**:
- false（默认）：按 queue + task_name 粒度返回，包含 tasks 数组
- true：按 queue 粒度返回，聚合所有 task_name，直接返回指标字段

**使用场景**: 监控面板、队列健康状态、任务性能分析""",
    responses={
        200: {
            "description": "成功返回任务执行汇总数据",
            "content": {
                "application/json": {
                    "examples": {
                        "按任务分组（默认）": {
                            "value": {
                                "success": True,
                                "data": [
                                    {
                                        "queue_name": "email_queue",
                                        "task_count": 2,
                                        "tasks": [
                                            {
                                                "task_name": "send_email",
                                                "total_count": 10000,
                                                "success_count": 9950,
                                                "failed_count": 50,
                                                "success_rate": 99.5
                                            }
                                        ]
                                    }
                                ],
                                "time_range": {
                                    "start_time": "2025-01-22T00:00:00Z",
                                    "end_time": "2025-01-22T23:59:59Z"
                                },
                                "pagination": {
                                    "next_cursor": None,
                                    "has_more": False
                                }
                            }
                        },
                        "按队列聚合（aggregate_tasks=true）": {
                            "value": {
                                "success": True,
                                "data": [
                                    {
                                        "queue_name": "email_queue",
                                        "total_count": 15000,
                                        "success_count": 14900,
                                        "failed_count": 100,
                                        "retry_count": 50,
                                        "success_rate": 99.33,
                                        "avg_duration": 1.25,
                                        "max_duration": 15.6,
                                        "avg_delay": 0.5,
                                        "max_delay": 5.2,
                                        "max_concurrency": 50
                                    }
                                ],
                                "time_range": {
                                    "start_time": "2025-01-22T00:00:00Z",
                                    "end_time": "2025-01-22T23:59:59Z"
                                }
                            }
                        }
                    }
                }
            }
        },
        400: {"description": "请求参数错误"},
        500: {"description": "服务器内部错误"}
    }
)
async def get_metrics_runs_summary(
    ns: NsDep,
    queues: str = Query(..., description="队列名称，多个用逗号分隔", example="email_queue,sms_queue"),
    task_names: Optional[str] = Query(None, description="任务名称筛选，多个用逗号分隔"),
    metrics: Optional[str] = Query(
        None,
        description="指定返回的指标，逗号分隔，默认返回全部。"
                    "支持: total_count, success_count, failed_count, retry_count, "
                    "avg_duration, max_duration, min_duration, avg_delay, max_delay, min_delay, "
                    "max_concurrency, success_rate",
        example="total_count,success_count,success_rate"
    ),
    start_time: Optional[datetime] = Query(None, description="开始时间（ISO格式）"),
    end_time: Optional[datetime] = Query(None, description="结束时间（ISO格式）"),
    time_range: Optional[str] = Query(None, description="时间范围，如 15m, 1h, 24h, 7d", example="1h"),
    aggregate_tasks: bool = Query(False, description="是否按队列聚合（忽略 task_name 维度）"),
    limit: int = Query(50, description="每页返回的任务数量", ge=1, le=200),
    cursor: Optional[str] = Query(None, description="分页游标")
) -> Dict[str, Any]:
    try:
        pg_session = await ns.get_pg_session()
        registry = await ns.get_queue_registry()

        try:
            queue_list = [q.strip() for q in queues.split(',') if q.strip()]
            if not queue_list:
                raise HTTPException(status_code=400, detail="queues 参数不能为空")

            task_name_list = None
            if task_names:
                task_name_list = [t.strip() for t in task_names.split(',') if t.strip()]

            metric_list = None
            if metrics:
                metric_list = []
                for m in metrics.split(','):
                    m = m.strip()
                    try:
                        metric_list.append(RunsMetric(m))
                    except ValueError:
                        raise HTTPException(
                            status_code=400,
                            detail=f"无效的指标名称: {m}。支持: {', '.join([e.value for e in RunsMetric])}"
                        )

            cursor_data = None
            if cursor:
                try:
                    cursor_data = json.loads(base64.b64decode(cursor).decode('utf-8'))
                except Exception:
                    raise HTTPException(status_code=400, detail="无效的 cursor")

            result = await MetricsService.get_runs_summary(
                namespace=ns.name,
                pg_session=pg_session,
                registry=registry,
                queues=queue_list,
                task_names=task_name_list,
                metrics=metric_list,
                start_time=start_time,
                end_time=end_time,
                time_range=time_range,
                aggregate_tasks=aggregate_tasks,
                limit=limit,
                cursor=cursor_data
            )
            return result

        finally:
            await pg_session.close()

    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"获取任务执行汇总失败: {e}", exc_info=True)
        raise HTTPException(status_code=500, detail=str(e))


@router.get(
    "/metrics/runs/trend",
    summary="任务执行时序趋势",
    description="""从 task_runs_metrics_minute 聚合表查询任务执行的时序趋势数据。

**功能说明**:
- 按时间粒度返回任务执行指标的时序数据
- 支持选择性聚合指标（节省算力）
- 时间间隔自动根据查询范围计算

**支持的指标**:
- 计数类：total_count, success_count, failed_count, retry_count
- 耗时类：avg_duration, max_duration, min_duration
- 延迟类：avg_delay, max_delay, min_delay
- 并发类：max_concurrency
- 计算类：success_rate, error_rate
- 分位数类：duration_percentiles (返回 duration_p50/p90/p99), delay_percentiles (返回 delay_p50/p90/p99)
- 积压类：backlog (返回 backlog, pending_read)

**时间间隔自动计算**:
- ≤ 1 小时：1 分钟
- 1-6 小时：5 分钟
- 6-24 小时：15 分钟
- 1-7 天：1 小时
- > 7 天：6 小时

**使用场景**: 执行速率监控、成功率趋势、延迟监控、并发量监控""",
    responses={
        200: {
            "description": "成功返回任务执行趋势数据",
            "content": {
                "application/json": {
                    "example": {
                        "success": True,
                        "data": [
                            {
                                "queue_name": "email_queue",
                                "trend": [
                                    {
                                        "time": "2025-01-22T10:00:00Z",
                                        "total_count": 100,
                                        "success_count": 98
                                    }
                                ]
                            }
                        ],
                        "metrics": ["total_count", "success_count"],
                        "granularity": "5m",
                        "time_range": {
                            "start_time": "2025-01-22T10:00:00Z",
                            "end_time": "2025-01-22T11:00:00Z",
                            "interval": "5 minutes",
                            "interval_seconds": 300
                        }
                    }
                }
            }
        },
        400: {"description": "请求参数错误"},
        500: {"description": "服务器内部错误"}
    }
)
async def get_metrics_runs_trend(
    ns: NsDep,
    queues: Optional[str] = Query(None, description="队列名称，多个用逗号分隔", example="email_queue,sms_queue"),
    queue: Optional[str] = Query(None, description="队列名称（单个，兼容旧版）", example="email_queue"),
    task_names: Optional[str] = Query(None, description="任务名称筛选，多个用逗号分隔"),
    metrics: Optional[str] = Query(
        None,
        description="指定聚合的指标，逗号分隔，默认 total_count。"
                    "支持: total_count, success_count, failed_count, retry_count, "
                    "avg_duration, max_duration, min_duration, avg_delay, max_delay, min_delay, "
                    "max_concurrency, success_rate, duration_percentiles, delay_percentiles, backlog, error_rate",
        example="total_count,success_count,duration_percentiles"
    ),
    start_time: Optional[datetime] = Query(None, description="开始时间（ISO格式）"),
    end_time: Optional[datetime] = Query(None, description="结束时间（ISO格式）"),
    time_range: Optional[str] = Query(None, description="时间范围，如 15m, 1h, 24h, 7d", example="1h")
) -> Dict[str, Any]:
    try:
        pg_session = await ns.get_pg_session()
        registry = await ns.get_queue_registry()

        try:
            queues_param = queues or queue
            if not queues_param:
                raise HTTPException(status_code=400, detail="queues 或 queue 参数不能为空")
            queue_list = [q.strip() for q in queues_param.split(',') if q.strip()]
            if not queue_list:
                raise HTTPException(status_code=400, detail="queues 或 queue 参数不能为空")

            task_name_list = None
            if task_names:
                task_name_list = [t.strip() for t in task_names.split(',') if t.strip()]

            metric_list = None
            if metrics:
                metric_list = []
                for m in metrics.split(','):
                    m = m.strip()
                    try:
                        metric_list.append(RunsMetric(m))
                    except ValueError:
                        raise HTTPException(
                            status_code=400,
                            detail=f"无效的指标名称: {m}。支持: {', '.join([e.value for e in RunsMetric])}"
                        )

            result = await MetricsService.get_runs_trend(
                namespace=ns.name,
                pg_session=pg_session,
                registry=registry,
                queues=queue_list,
                task_names=task_name_list,
                metrics=metric_list,
                start_time=start_time,
                end_time=end_time,
                time_range=time_range
            )
            return result

        finally:
            await pg_session.close()

    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"获取任务执行趋势失败: {e}", exc_info=True)
        raise HTTPException(status_code=500, detail=str(e))




@router.get(
    "/runs",
    summary="任务执行记录列表",
    description="""从 tasks + task_runs 表联查，获取任务执行的详细记录。

**功能说明**:
- 支持复杂的 SQL 条件筛选
- 使用 cursor 分页，适合大数据量场景
- 返回轻量级数据，不包含 payload/result/error 等大字段
- 自动将基础队列名展开为所有优先级队列

**查询条件**:
- queue: 必填，单个队列名称
- start_time/end_time/time_range: 时间范围
- status: 状态筛选，多个用逗号分隔
- task_names: 任务名称筛选，多个用逗号分隔
- where_clause: 自定义 SQL 条件（高级功能）

**where_clause 可用字段**:
- stream_id, priority, created_at, trigger_time, source
- task_name, status, started_at, completed_at, retries, duration, consumer

**where_clause 示例**:
- `stream_id = '1769116820603'`
- `status = 'failed' AND duration > 10`
- `source = 'api' AND consumer LIKE 'worker-1%'`

**使用场景**: 任务执行历史查询、问题排查、性能分析""",
    responses={
        200: {
            "description": "成功返回任务执行记录",
            "content": {
                "application/json": {
                    "example": {
                        "success": True,
                        "data": [
                            {
                                "stream_id": "1769065763365-0",
                                "task_name": "send_notification",
                                "status": "success",
                                "priority": 1,
                                "retries": 0,
                                "created_at": "2026-01-22T10:00:00Z",
                                "started_at": "2026-01-22T10:00:01Z",
                                "completed_at": "2026-01-22T10:00:02Z",
                                "duration": 1.05,
                                "delay": 1.0,
                                "consumer": "worker-1",
                                "source": "api"
                            }
                        ],
                        "queue": "email_queue",
                        "time_range": {
                            "start_time": "2026-01-22T09:00:00Z",
                            "end_time": "2026-01-22T10:00:00Z"
                        },
                        "pagination": {
                            "next_cursor": "eyJzdHJlYW1faWQiOiIxNzY5MDY1NzYzMzY1LTAifQ==",
                            "has_more": True,
                            "estimated_total": 1234
                        }
                    }
                }
            }
        },
        400: {"description": "请求参数错误"},
        500: {"description": "服务器内部错误"}
    }
)
async def get_task_runs_list(
    ns: NsDep,
    queue: str = Query(..., description="队列名称（必填）", example="email_queue"),
    start_time: Optional[datetime] = Query(None, description="开始时间（ISO格式）"),
    end_time: Optional[datetime] = Query(None, description="结束时间（ISO格式）"),
    time_range: Optional[str] = Query(None, description="时间范围，如 1h, 24h, 7d", example="1h"),
    status: Optional[str] = Query(None, description="状态筛选，多个用逗号分隔。支持: pending, running, success, error, failed, delayed, rejected, retry, timeout, cancelled, scheduled", example="success,failed"),
    task_names: Optional[str] = Query(None, description="任务名称筛选，多个用逗号分隔"),
    where_clause: Optional[str] = Query(None, description="自定义 SQL WHERE 条件，如 status = 'failed' AND duration > 10"),
    limit: int = Query(50, description="每页数量", ge=1, le=200),
    cursor: Optional[str] = Query(None, description="分页游标"),
    sort_field: str = Query("created_at", description="排序字段"),
    sort_order: str = Query("desc", description="排序方向", regex="^(asc|desc)$"),
    include_fields: Optional[str] = Query(None, description="额外返回的大字段，多个用逗号分隔。支持: payload, result, error, metadata", example="error,payload")
) -> Dict[str, Any]:
    try:
        pg_session = await ns.get_pg_session()
        registry = await ns.get_queue_registry()

        try:
            status_list = None
            if status:
                status_list = [s.strip() for s in status.split(',') if s.strip()]

            task_name_list = None
            if task_names:
                task_name_list = [t.strip() for t in task_names.split(',') if t.strip()]

            include_field_list = None
            if include_fields:
                valid_fields = {'payload', 'result', 'error', 'metadata'}
                include_field_list = [f.strip() for f in include_fields.split(',') if f.strip() in valid_fields]

            result = await QueueService.get_task_runs_list(
                namespace=ns.name,
                pg_session=pg_session,
                registry=registry,
                queue=queue,
                start_time=start_time,
                end_time=end_time,
                time_range=time_range,
                status=status_list,
                task_names=task_name_list,
                where_clause=where_clause,
                limit=limit,
                cursor=cursor,
                sort_field=sort_field,
                sort_order=sort_order,
                include_fields=include_field_list
            )
            return result

        finally:
            await pg_session.close()

    except ValueError as e:
        raise HTTPException(status_code=400, detail=str(e))
    except Exception as e:
        logger.error(f"获取任务执行记录失败: {e}", exc_info=True)
        raise HTTPException(status_code=500, detail=str(e))


@router.get(
    "/runs/trend",
    summary="任务执行趋势（支持复杂条件和维度聚合）",
    description="""从 tasks + task_runs 表联查，获取任务执行趋势数据。

**功能说明**:
- 支持与 /runs 接口相同的查询条件
- 按时间桶聚合统计 total_count, success_count, failed_count
- 支持按指定维度分组聚合（group_by 参数）
- 时间粒度自动根据查询范围计算

**时间聚合说明**:
- 返回的 time 字段表示任务**开始执行时间**（started_at），而非入队时间（created_at）
- 每个时间桶内的统计是该时间段内开始执行的任务数量

**聚合维度（group_by）** - 支持多个，用逗号分隔:
- priority: 按优先级分组
- source: 按来源分组
- task_name: 按任务名称分组
- consumer: 按消费者分组
- status: 按状态分组

示例: `group_by=task_name,priority` 按任务名称和优先级组合分组

**where_clause 可用字段**:
- stream_id, priority, created_at, trigger_time, source
- task_name, status, started_at, completed_at, retries, duration, consumer

**where_clause 示例**:
- `stream_id = '1769116820603'`
- `status = 'failed' AND duration > 10`

**与 /metrics/runs/trend 的区别**:
- /metrics/runs/trend: 从预聚合表查询，查询快，但无法支持灵活的 SQL 条件
- /runs/trend: 从原始表查询，支持任意 SQL 条件和维度聚合，适合精确筛选场景

**使用场景**: 配合 /runs 接口使用，在同一页面展示符合条件的趋势图和任务列表""",
    responses={
        200: {
            "description": "成功返回任务执行趋势",
            "content": {
                "application/json": {
                    "examples": {
                        "无分组": {
                            "value": {
                                "success": True,
                                "data": {
                                    "trend": [
                                        {"time": "2026-01-22T10:00:00Z", "total_count": 100, "success_count": 95, "failed_count": 5},
                                        {"time": "2026-01-22T10:05:00Z", "total_count": 120, "success_count": 118, "failed_count": 2}
                                    ]
                                },
                                "queue": "email_queue",
                                "group_by": None,
                                "granularity": "5m",
                                "time_range": {
                                    "start_time": "2026-01-22T10:00:00Z",
                                    "end_time": "2026-01-22T11:00:00Z"
                                }
                            }
                        },
                        "按 task_name 分组": {
                            "value": {
                                "success": True,
                                "data": {
                                    "send_email": {
                                        "dimensions": {"task_name": "send_email"},
                                        "trend": [
                                            {"time": "2026-01-22T10:00:00Z", "total_count": 50, "success_count": 48, "failed_count": 2}
                                        ]
                                    },
                                    "send_sms": {
                                        "dimensions": {"task_name": "send_sms"},
                                        "trend": [
                                            {"time": "2026-01-22T10:00:00Z", "total_count": 50, "success_count": 47, "failed_count": 3}
                                        ]
                                    }
                                },
                                "queue": "email_queue",
                                "group_by": ["task_name"],
                                "granularity": "5m"
                            }
                        },
                        "按 task_name 和 priority 多维度分组": {
                            "value": {
                                "success": True,
                                "data": {
                                    "send_email|1": {
                                        "dimensions": {"task_name": "send_email", "priority": "1"},
                                        "trend": [
                                            {"time": "2026-01-22T10:00:00Z", "total_count": 30, "success_count": 28, "failed_count": 2}
                                        ]
                                    },
                                    "send_email|2": {
                                        "dimensions": {"task_name": "send_email", "priority": "2"},
                                        "trend": [
                                            {"time": "2026-01-22T10:00:00Z", "total_count": 20, "success_count": 20, "failed_count": 0}
                                        ]
                                    }
                                },
                                "queue": "email_queue",
                                "group_by": ["task_name", "priority"],
                                "granularity": "5m"
                            }
                        }
                    }
                }
            }
        },
        400: {"description": "请求参数错误"},
        500: {"description": "服务器内部错误"}
    }
)
async def get_task_runs_trend(
    ns: NsDep,
    queue: str = Query(..., description="队列名称（必填）", example="email_queue"),
    start_time: Optional[datetime] = Query(None, description="开始时间（ISO格式）"),
    end_time: Optional[datetime] = Query(None, description="结束时间（ISO格式）"),
    time_range: Optional[str] = Query(None, description="时间范围，如 1h, 24h, 7d", example="1h"),
    status: Optional[str] = Query(None, description="状态筛选，多个用逗号分隔。支持: pending, running, success, error, failed, delayed, rejected, retry, timeout, cancelled, scheduled", example="success,failed"),
    task_names: Optional[str] = Query(None, description="任务名称筛选，多个用逗号分隔"),
    where_clause: Optional[str] = Query(None, description="自定义 SQL WHERE 条件，如 status = 'failed' AND duration > 10"),
    group_by: Optional[str] = Query(
        None,
        description="聚合维度，多个用逗号分隔。支持: priority, source, task_name, consumer, status",
        example="task_name,priority"
    )
) -> Dict[str, Any]:
    try:
        pg_session = await ns.get_pg_session()
        registry = await ns.get_queue_registry()

        try:
            status_list = None
            if status:
                status_list = [s.strip() for s in status.split(',') if s.strip()]

            task_name_list = None
            if task_names:
                task_name_list = [t.strip() for t in task_names.split(',') if t.strip()]

            group_by_list = None
            if group_by:
                group_by_list = [g.strip() for g in group_by.split(',') if g.strip()]

            result = await QueueService.get_task_runs_trend(
                namespace=ns.name,
                pg_session=pg_session,
                registry=registry,
                queue=queue,
                start_time=start_time,
                end_time=end_time,
                time_range=time_range,
                status=status_list,
                task_names=task_name_list,
                where_clause=where_clause,
                group_by=group_by_list
            )
            return result

        finally:
            await pg_session.close()

    except ValueError as e:
        raise HTTPException(status_code=400, detail=str(e))
    except Exception as e:
        logger.error(f"获取任务执行趋势失败: {e}", exc_info=True)
        raise HTTPException(status_code=500, detail=str(e))


@router.get(
    "/runs/{stream_id}",
    summary="任务执行详情",
    description="""获取单个任务执行记录的完整详情。

**功能说明**:
- 包含 payload、result、error 等大字段
- 适合查看任务的完整执行信息

**使用场景**: 任务详情页面、错误排查、执行结果查看""",
    responses={
        200: {
            "description": "成功返回任务详情",
            "content": {
                "application/json": {
                    "example": {
                        "success": True,
                        "data": {
                            "stream_id": "1769065763365-0",
                            "queue": "email_queue",
                            "task_name": "send_notification",
                            "status": "success",
                            "payload": {"to": "user@example.com", "subject": "Hello"},
                            "result": {"message_id": "abc123"},
                            "error": None,
                            "priority": 1,
                            "retries": 0,
                            "created_at": "2026-01-22T10:00:00Z",
                            "started_at": "2026-01-22T10:00:01Z",
                            "completed_at": "2026-01-22T10:00:02Z",
                            "duration": 1.05,
                            "delay_seconds": 1.0,
                            "consumer": "worker-1",
                            "source": "api"
                        }
                    }
                }
            }
        },
        404: {"description": "任务不存在"},
        500: {"description": "服务器内部错误"}
    }
)
async def get_task_run_detail(
    ns: NsDep,
    stream_id: str = Path(..., description="任务 Stream ID")
) -> Dict[str, Any]:
    try:
        pg_session = await ns.get_pg_session()

        try:
            result = await QueueService.get_task_run_detail(
                namespace=ns.name,
                pg_session=pg_session,
                stream_id=stream_id
            )

            if not result.get("success") and "不存在" in result.get("error", ""):
                raise HTTPException(status_code=404, detail=result.get("error"))

            return result

        finally:
            await pg_session.close()

    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"获取任务详情失败: {e}", exc_info=True)
        raise HTTPException(status_code=500, detail=str(e))




@router.post(
    "/send-queue",
    summary="发送任务到队列",
    description="""跨语言的任务发送接口，支持从任何编程语言通过 HTTP API 发送任务。

**功能特性**: 支持批量发送多个任务、设置任务优先级、延迟执行、返回每个任务的事件 ID

**请求体参数**: messages 数组，每个 message 包含 queue（必需）、priority（可选，1最高）、delay（可选，秒）、kwargs（字典）、args（列表）

**使用场景**: Go/Java 服务发送任务到 Python worker、微服务跨语言任务调度、外部系统集成（Webhook、第三方服务）、定时任务触发器、批量数据处理任务

**注意**: 命名空间必须存在，priority: 1 是最高优先级，返回的 event_ids 可用于追踪任务状态""",
    response_model=SendTasksResponse,
    responses={
        200: {
            "description": "任务发送成功",
            "content": {
                "application/json": {
                    "example": {
                        "success": True,
                        "message": "Tasks sent successfully",
                        "event_ids": [
                            "1730361234567-0",
                            "1730361234568-0"
                        ],
                        "total_sent": 2,
                        "namespace": "default"
                    }
                }
            }
        },
        400: {"description": "请求参数错误"},
        404: {"description": "命名空间不存在"},
        500: {"description": "服务器内部错误"}
    }
)
async def send_tasks(
    request: Request,
    send_request: SendTasksRequest,
    ns: NsDep
) -> SendTasksResponse:
    try:
        namespace = ns.name

        client_host = request.client.host if request.client else 'unknown'
        client_port = request.client.port if request.client else 'unknown'

        with LogContext(
            endpoint='/queues/send-queue',
            namespace=namespace,
            client_ip=client_host,
            client_port=client_port,
            task_count=len(send_request.messages)
        ):
            logger.info(
                f"收到任务发送请求",
                extra={
                    'extra_fields': {
                        'namespace': namespace,
                        'task_count': len(send_request.messages),
                        'client_ip': client_host,
                        'client_port': client_port,
                        'request_path': str(request.url.path),
                        'method': request.method,
                        'messages': [msg.model_dump() for msg in send_request.messages ]
                    }
                }
            )

            for idx, msg_req in enumerate(send_request.messages):
                logger.debug(
                    f"任务 #{idx+1} 详情",
                    extra={
                        'extra_fields': {
                            'task_index': idx + 1,
                            'queue': msg_req.queue,
                            'priority': msg_req.priority,
                            'delay': msg_req.delay,
                            'has_args': bool(msg_req.args),
                            'args_count': len(msg_req.args) if msg_req.args else 0,
                            'kwargs_keys': list(msg_req.kwargs.keys()) if msg_req.kwargs else [],
                            'kwargs': msg_req.kwargs
                        }
                    }
                )

            logger.debug(f"正在获取命名空间 '{namespace}' 的 Jettask 实例")
            jettask_app = await ns.get_jettask_app()

            task_messages = []
            for msg_req in send_request.messages:
                msg = TaskMessage(
                    queue=msg_req.queue,
                    kwargs=msg_req.kwargs,
                    priority=msg_req.priority,
                    delay=msg_req.delay,
                    args=tuple(msg_req.args) if msg_req.args else ()
                )
                task_messages.append(msg)

            logger.info(
                f"开始发送 {len(task_messages)} 个任务到命名空间 '{namespace}'"
            )
            event_ids = await jettask_app.send_tasks(task_messages, asyncio=True)

            if not event_ids:
                logger.error(
                    "任务发送失败: 未返回事件ID",
                    extra={
                        'extra_fields': {
                            'namespace': namespace,
                            'task_count': len(task_messages)
                        }
                    }
                )
                raise HTTPException(
                    status_code=500,
                    detail="Failed to send tasks: no event IDs returned"
                )

            logger.info(
                f"成功发送 {len(event_ids)} 个任务到命名空间 '{namespace}'",
                extra={
                    'extra_fields': {
                        'namespace': namespace,
                        'event_ids': event_ids,
                        'total_sent': len(event_ids),
                        'client_ip': client_host
                    }
                }
            )

            return SendTasksResponse(
                success=True,
                message="Tasks sent successfully",
                event_ids=event_ids,
                total_sent=len(event_ids),
                namespace=namespace
            )

    except HTTPException:
        raise
    except Exception as e:
        namespace_name = getattr(request.state, 'ns', None)
        namespace_name = namespace_name.name if namespace_name else 'unknown'

        logger.error(
            f"任务发送失败",
            extra={
                'extra_fields': {
                    'namespace': namespace_name,
                    'error': str(e),
                    'error_type': type(e).__name__
                }
            },
            exc_info=True
        )

        return SendTasksResponse(
            success=False,
            message="Failed to send tasks",
            event_ids=[],
            total_sent=0,
            namespace=namespace_name,
            error=str(e)
        )




@router.get(
    "/{queue}/tasks",
    summary="获取队列下的任务列表",
    description="""获取指定队列下注册的所有任务名称。

**数据来源**:
- 从 REGISTRY:TASKS Hash 读取（Worker 启动订阅队列时注册）
- 比 READ_OFFSETS 更可靠：即使任务未消费过，只要 Worker 启动过就能获取

**注意**: 如果 Worker 从未启动过，该队列下不会有任务记录。

**使用场景**: 任务配置管理、任务选择器、队列任务概览""",
    responses={
        200: {
            "description": "成功返回任务列表",
            "content": {
                "application/json": {
                    "example": {
                        "success": True,
                        "namespace": "default",
                        "queue": "email_queue",
                        "tasks": ["send_email", "send_notification"],
                        "total": 2
                    }
                }
            }
        },
        500: {"description": "服务器内部错误"}
    }
)
async def get_queue_tasks(
    ns: NsDep,
    queue: str = Path(..., description="队列名称", example="email_queue"),
    exclude_internal: bool = Query(False, description="是否排除内部任务（如 _handle_persist_task）")
) -> Dict[str, Any]:
    try:
        registry = await ns.get_queue_registry()
        tasks = await registry.get_tasks_by_queue(queue)

        if exclude_internal:
            tasks = [t for t in tasks if not t.endswith('._handle_persist_task')]

        return {
            "success": True,
            "namespace": ns.name,
            "queue": queue,
            "tasks": sorted(tasks),
            "total": len(tasks)
        }
    except Exception as e:
        logger.error(f"获取队列任务列表失败: {e}", exc_info=True)
        raise HTTPException(status_code=500, detail=str(e))



from pydantic import BaseModel, Field


class TaskConfigResponse(BaseModel):
    """任务配置响应"""
    success: bool = True
    data: Dict[str, Any]


class UpdateTaskConfigRequest(BaseModel):
    """更新任务配置请求

    注意：priority（任务优先级）不在此处配置，优先级只能在发送任务时由客户端指定
    """
    rate_limit: Optional[Dict[str, Any]] = Field(None, description="限流配置，如 {type: 'qps', qps: 10, window_size: 1.0} 或 {type: 'concurrency', max_concurrency: 5}")
    max_retries: Optional[int] = Field(None, description="最大重试次数")
    retry_delay: Optional[int] = Field(None, description="重试延迟（秒）")


@router.get(
    "/{queue}/tasks/{task_name}/config",
    summary="获取任务配置",
    description="""获取指定任务的动态配置。

**配置项**:
- `rate_limit`: 限流配置（QPS 或并发限流，二选一）
- `max_retries`: 最大重试次数
- `retry_delay`: 重试延迟（秒）

**注意**: 任务优先级（priority）不在此处配置，优先级只能在发送任务时由客户端指定

**使用场景**: 任务详情页面、配置管理""",
    response_model=TaskConfigResponse,
)
async def get_task_config(
    request: Request,
    ns: NsDep,
    queue: str = Path(..., description="队列名称"),
    task_name: str = Path(..., description="任务名称")
) -> Dict[str, Any]:
    from jettask.config import DynamicConfig

    try:
        redis_client = await ns.get_redis_client()
        prefix = ns.redis_prefix

        config_key = DynamicConfig.get_task_config_key(prefix, task_name)
        data = await redis_client.hgetall(config_key)

        configs = {}
        for k, v in data.items():
            k = k.decode() if isinstance(k, bytes) else k
            v = v.decode() if isinstance(v, bytes) else v
            configs[k] = v

        return {
            "success": True,
            "data": {
                "namespace": ns.name,
                "queue": queue,
                "task_name": task_name,
                "configs": configs
            }
        }
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"获取任务配置失败: {e}", exc_info=True)
        raise HTTPException(status_code=500, detail=str(e))


@router.put(
    "/{queue}/tasks/{task_name}/config",
    summary="更新任务配置",
    description="""更新指定任务的动态配置。

配置会同步到对应命名空间的 Redis，Worker 会在下次同步时获取新配置。

**注意**:
- 配置只对新任务生效
- 已在队列中的任务不受影响
- Worker 同步延迟约 30 秒

**示例请求**:
```json
{
    "rate_limit": {"type": "qps", "qps": 10, "window_size": 1.0},
    "max_retries": 5,
    "retry_delay": 60
}
```

**限流配置说明**:
- QPS 限流: `{"type": "qps", "qps": 10, "window_size": 1.0}`
- 并发限流: `{"type": "concurrency", "max_concurrency": 5}`
- 每个任务只能配置一种限流类型""",
    response_model=TaskConfigResponse,
)
async def update_task_config(
    request: Request,
    ns: NsDep,
    queue: str = Path(..., description="队列名称"),
    task_name: str = Path(..., description="任务名称"),
    body: UpdateTaskConfigRequest = None
) -> Dict[str, Any]:
    from jettask.config import DynamicConfig
    from jettask.webui.services.config_service import ConfigService

    try:
        configs = {}
        if body:
            logger.info(f"收到任务配置更新请求: task={task_name}, body={body.model_dump()}")
            if body.rate_limit is not None:
                configs['rate_limit'] = body.rate_limit
            if body.max_retries is not None:
                configs['max_retries'] = body.max_retries
            if body.retry_delay is not None:
                configs['retry_delay'] = body.retry_delay

        if not configs:
            raise HTTPException(status_code=400, detail="至少需要提供一个配置项")

        redis_client = await ns.get_redis_client()
        prefix = ns.redis_prefix

        await DynamicConfig.set_task_config_to_redis(
            redis_client, prefix, task_name, configs
        )

        config_key = DynamicConfig.get_task_config_key(prefix, task_name)
        redis_task_version = await redis_client.hget(config_key, '_version')
        redis_task_version = int(redis_task_version) if redis_task_version else 0

        pg_session = await ns.get_pg_session()
        try:
            await ConfigService.save_task_config_to_pg(
                pg_session, ns.name, queue, task_name, configs
            )
            await ConfigService.set_task_version_in_pg(
                pg_session, ns.name, queue, task_name, redis_task_version
            )
        finally:
            await pg_session.close()

        return {
            "success": True,
            "data": {
                "namespace": ns.name,
                "queue": queue,
                "task_name": task_name,
                "configs": configs
            }
        }
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"更新任务配置失败: {e}", exc_info=True)
        raise HTTPException(status_code=500, detail=str(e))


@router.delete(
    "/{queue}/tasks/{task_name}/config",
    summary="删除任务配置",
    description="删除指定任务的动态配置，恢复使用代码中的默认值。",
)
async def delete_task_config(
    request: Request,
    ns: NsDep,
    queue: str = Path(..., description="队列名称"),
    task_name: str = Path(..., description="任务名称")
) -> Dict[str, Any]:
    from jettask.config import DynamicConfig
    from jettask.webui.services.config_service import ConfigService

    try:
        redis_client = await ns.get_redis_client()
        prefix = ns.redis_prefix

        await DynamicConfig.delete_task_config_from_redis(
            redis_client, prefix, task_name
        )

        pg_session = await ns.get_pg_session()
        try:
            await ConfigService.delete_task_config_from_pg(
                pg_session, ns.name, queue, task_name
            )
        finally:
            await pg_session.close()

        return {
            "success": True,
            "message": f"任务配置已删除: {ns.name}:{queue}:{task_name}"
        }
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"删除任务配置失败: {e}", exc_info=True)
        raise HTTPException(status_code=500, detail=str(e))


@router.get(
    "/{queue}/task-runs/{stream_id}",
    summary="获取任务执行详情",
    description="""获取单个任务的详细信息，包含 payload、result、error 等大字段。

**使用场景**: 在列表页点击某条任务后，加载完整详情。
列表接口不返回这些大字段以保证性能，需要通过此接口单独获取。

**可选字段**: 通过 `fields` 参数指定需要返回的字段，不指定则返回全部。
支持的字段: payload, result, error, metadata""",
    responses={
        200: {
            "description": "成功返回任务详情",
            "content": {
                "application/json": {
                    "example": {
                        "success": True,
                        "data": {
                            "stream_id": "1737945600000-0",
                            "queue": "email_queue",
                            "task_name": "send_email",
                            "status": "success",
                            "payload": {"to": "user@example.com"},
                            "result": {"message_id": "abc123"},
                            "error": None
                        }
                    }
                }
            }
        },
        404: {"description": "任务不存在"},
        500: {"description": "服务器内部错误"}
    }
)
async def get_task_run_detail(
    ns: NsDep,
    queue: str = Path(..., description="队列名称"),
    stream_id: str = Path(..., description="任务 Stream ID"),
    fields: Optional[List[str]] = Query(None, description="指定返回的大字段列表，如 payload, result, error, metadata")
) -> Dict[str, Any]:
    try:
        session = await ns.get_pg_session()
        async with session:
            result = await QueueService.get_task_run_detail(
                namespace=ns.name,
                pg_session=session,
                stream_id=stream_id
            )

        if not result.get("success"):
            raise HTTPException(status_code=404, detail=result.get("error", "任务不存在"))

        if fields and result.get("data"):
            large_fields = {"payload", "result", "error", "metadata"}
            requested = set(fields)
            for f in large_fields:
                if f not in requested and f in result["data"]:
                    result["data"][f] = None

        return result
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"获取任务详情失败: stream_id={stream_id}, error={e}", exc_info=True)
        raise HTTPException(status_code=500, detail=str(e))


__all__ = ['router']